<?php
class file extends \SplFileInfo
{

    public function __construct($path, $checkPath = true)
    {
        if ($checkPath && !is_file($path)) {
            throw new \RuntimeException($path);
        }

        parent::__construct($path);

    }

    public function move($directory, $name = null)
    {

        $target = $this->getTargetFile($directory, $name);

        set_error_handler(function ($type, $msg) use (&$error) { $error = $msg; });
        $moved = rename($this->getPathname(), $target);
        restore_error_handler();
        if (!$moved) {
            throw new \RuntimeException(sprintf('Could not move the file "%s" to "%s" (%s).', $this->getPathname(), $target, strip_tags($error)));
        }

        return $target;

    }

    public function getContent()
    {
        $content = file_get_contents($this->getPathname());

        if (false === $content) {
            throw new \RuntimeException(sprintf('Could not get the content of the file "%s".', $this->getPathname()));
        }

        return $content;
    }

    protected function getTargetFile($directory, $name = null)
    {
        if (!is_dir($directory)) {
            if (false === @mkdir($directory, 0777, true) && !is_dir($directory)) {
                throw new \RuntimeException(sprintf('Unable to create the "%s" directory.', $directory));
            }
        } elseif (!is_writable($directory)) {
            throw new \RuntimeException(sprintf('Unable to write in the "%s" directory.', $directory));
        }

        $target = rtrim($directory, '/\\').\DIRECTORY_SEPARATOR.(null === $name ? $this->getBasename() : $this->getName($name));

        return new self($target, false);
    }

    protected function getName($name)
    {
        $originalName = str_replace('\\', '/', $name);
        $pos = strrpos($originalName, '/');
        $originalName = false === $pos ? $originalName : substr($originalName, $pos + 1);

        return $originalName;
    }

}