<?php
use Shop\Entity\Order;
use Shop\Usecase\GetOrderByIdUsecase;
use Shop\Usecase\RunEventAndSaveOrderUsecase;
use Shop\Exception\NotFoundException;

class paymentAlfabank_settings_mapper
{
    private $createOrderUrl = 'https://pay.alfabank.ru/payment/rest/';
    private $testOrderUrl = 'https://web.rbsuat.com/ab/rest/';

    public function success()
    {
        header("Content-type: text/xml; charset=utf-8");

        if (!isset($_GET['orderNumber']) || !isset($_GET['mdOrder'])) {
            throw new NotFoundException('Not found');
        }

        $orderId = (int) $_GET['orderNumber'];

        $ordUsecase = new GetOrderByIdUsecase();
        $ord2 = $ordUsecase->run($orderId);

        if ($ord2->event() !== 'submit' && $ord2->event() !== 'work' && $ord2->event() !== 'cancel')
        {
            return;
        }

        if (!$ord2->payment()->id() || $ord2->payment()->status() == 'off') {
            throw new NotFoundException('Not found');
        }

        includeLangConst($ord2->langId());

        $data = [
            'userName' => $ord2->payment()->settings('userName'),
            'password' => $ord2->payment()->settings('password'),
            'orderId' => $_GET['mdOrder']
        ];

        if ($ord2->payment()->status() == 'test') {
            $this->createOrderUrl = $this->testOrderUrl;
        }

        try
        {
            $response = $this->gateway($this->createOrderUrl.'getOrderStatusExtended.do', $data);
        }
        catch (\Throwable $e)
        {
            throw new NotFoundException('bad');
        }

        if (!empty($response['errorCode']) ||
            !$response ||
            $response['orderNumber'] != $ord2->id() ||
            (string) ($ord2->price() * 100) != (string) $response['amount'] ||
            $response['amount'] <= 0)
        {
            throw new NotFoundException('bad');
        }

        if ($response['orderStatus'] != '1' && $response['orderStatus'] != '2') {
            throw new NotFoundException('bad');
        }

        $orderEvent = new RunEventAndSaveOrderUsecase($ord2);
        $orderEvent->run('paid');
    }

	public function getPaymentUrl(Order $ord2)
    {
        $quantity = (float) $ord2->productQuantity();
        $orderName = str_replace(array("\r", "\n", '+', "%"), array('', ' ', ' ', ''), $ord2->name());
        $successURL = $ord2->payment()->settings('successURL');
        $data = [
            'userName' => $ord2->payment()->settings('userName'),
            'password' => $ord2->payment()->settings('password'),
            'orderNumber' => $ord2->id(),
            'amount' => (string) ($ord2->price() * 100),
            'description' => mb_substr($orderName, 0, 99, 'UTF-8'),
            'returnUrl' => $successURL ? $successURL : siteProtocol.'://'.DOMAIN.BASE_DIR,
            'failUrl' => siteProtocol.'://'.DOMAIN.BASE_DIR
        ];
        //    'currency' => '643',//fix
        //    'language' => CUR_LANG,//fix

        $sno = $ord2->payment()->settings('sno');
        if ($sno !== '') {
            $data['taxSystem'] = $sno;
        }

        $tax = $ord2->payment()->settings('tax');
        if ($tax !== '' || $quantity > 1)
        {
	        $OrderBundle = array(
	        'orderCreationDate' => time(),
	        'cartItems' => array('items' => array(array(
	        'positionId' => 1,
	        'name' => mb_substr($ord2->productName(), 0, 99, 'UTF-8'),
	        'quantity' => array('value' => $quantity, 'measure' => 'шт'),
            'itemPrice' => (string) (round($ord2->price() / $quantity, 2) * 100)
	        ))));
            if ($tax !== '') {
                $OrderBundle['cartItems']['items'][0]['tax'] = array('taxType' => $tax);
            }
            $data['orderBundle'] = json_encode($OrderBundle);
	    }
	    if ($ord2->payment()->status() == 'test') {
            $this->createOrderUrl = $this->testOrderUrl;
	    }

        try
        {
            $response = $this->gateway($this->createOrderUrl.'register.do', $data);
        }
        catch (\Throwable $e)
        {
            throw new NotFoundException($e->getMessage());
        }

        if (!$response || !empty($response['errorCode']))
        {
            throw new NotFoundException('Alfabank error'.(!empty($response['errorCode']) ? ': '.$response['errorMessage'] : ''));
        }

        return $response['formUrl'];
    }

    public function settings($p)
    {
        if (!isset($p['settings']['userName']))
        {
            $p['settings']['userName'] = '';
        }

        if (!isset($p['settings']['password']))
        {
            $p['settings']['password'] = '';
        }

        if (!isset($p['settings']['tax']))
        {
            $p['settings']['tax'] = '';
        }

        if (!isset($p['settings']['sno']))
        {
            $p['settings']['sno'] = '';
        }

        if (!isset($p['settings']['successURL']))
        {
            $p['settings']['successURL'] = '';
        }

        return $p;
    }

    public function update($data)
    {
        return $data;
    }

    public function gateway($url, $data) {
        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_POST => true,
            CURLOPT_POSTFIELDS => http_build_query($data)
        ));
        $response = curl_exec($curl);
        $response = json_decode($response, true);
        $err = curl_error($curl);
        if ($err)
        {
            throw new NotFoundException('Curl error: '.$err);
        }
        curl_close($curl);
        return $response;
    }
}