<?php
use Shop\Entity\Order;
use Shop\Usecase\GetOrderByIdUsecase;
use Shop\Usecase\RunEventAndSaveOrderUsecase;
use Shop\Repository\SqlOrderRepository;
use Shop\Exception\NotFoundException;

class paymentBlockchain_settings_mapper
{
    private $createOrderUrl = 'https://api.blockchain.info/v2/receive';

    public function success()
    {
        header('Content-Type: text/html; charset=utf-8');

    	if (!isset($_REQUEST['address']) ||
            !isset($_REQUEST['secret']) ||
            !isset($_REQUEST['confirmations']) ||
            !isset($_REQUEST['invoice_id']))
        {
            throw new NotFoundException('Error request');
        }

    	if (isset($_REQUEST['test']) && $_REQUEST['test'] == true) {
            throw new NotFoundException('Ignoring Test Callback');
		}

        $orderId = (int) $_REQUEST['invoice_id'];

        $ordUsecase = new GetOrderByIdUsecase();
        $ord2 = $ordUsecase->run($orderId);

        if ($ord2->event() !== 'submit' && $ord2->event() !== 'work' && $ord2->event() !== 'cancel')
        {
            print '*ok*';
            return;
        }

        if (!$ord2->payment()->id() ||
            $ord2->payment()->status() == 'off' ||
            $_REQUEST['secret'] != $ord2->payment()->settings('secret'))
        {
            throw new NotFoundException('bad');
        }

        includeLangConst($ord2->langId());

        $value_in_btc = $_REQUEST['value'] / 100000000;
        $price_in_btc = $ord2->orderInfo('price_in_btc');
        if (!$price_in_btc || $price_in_btc['value'] != $value_in_btc) {
            print 'bad';
            return;
        }

        $address = $ord2->orderInfo('address');
		if ($_REQUEST['address'] != $address['value']) {
            print 'Incorrect Receiving Address';
            return;
        }

        if ($_REQUEST['confirmations'] < $ord2->payment()->settings('confirmations'))
        {
            $ord2->updateOrderInfo('confirmations', [
                'baseName' => Confirmations.' (BTC)',
                'value' => $_REQUEST['confirmations']
            ]);

            $repo = new SqlOrderRepository();
            $repo->save($ord2);

        	print 'Waiting for confirmations';
        }
        else
        {
            if (!empty($_REQUEST['transaction_hash'])) {
                $ord2->updateOrderInfo('transaction_hash', [
                    'baseName' => 'Transaction hash (BTC)',
                    'value' => $_REQUEST['transaction_hash']
                ]);
            }

            $orderEvent = new RunEventAndSaveOrderUsecase($ord2);
            $orderEvent->run('paid');
            print '*ok*';
        }

    }

    public function getPaymentUrl(Order $ord2)
    {
    	if ($ord2->payment()->currency() != 'BTC') {
          $price = cp_get_contents('https://blockchain.info/tobtc?currency='.$ord2->payment()->currency().'&value='.$ord2->price());
    	} else {
          $price = $ord2->price();
    	}

        if (!$price)
        {
            throw new NotFoundException('Error exchange BTC');
    	}

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
        curl_setopt($ch, CURLOPT_HEADER, false);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 30);
        curl_setopt($ch, CURLOPT_TIMEOUT, 60);
        curl_setopt($ch, CURLOPT_URL, $this->createOrderUrl.
        '?callback='.urlencode(siteProtocol.'://'.DOMAIN.BASE_DIR.'/payment/Blockchain/callback.php?invoice_id='.$ord2->id().
        '&secret='.$ord2->payment()->settings('secret')).
        '&xpub='.$ord2->payment()->settings('bitcoinAddress').
        '&key='.$ord2->payment()->settings('apikey'));

        $response = curl_exec($ch);
        if ($response) $response = json_decode($response, true);
        curl_close($ch);

        if (empty($response['address']))
        {
            $message = 'Error blockchain connect';
            if (isset($response['message'])) {
                $message = $response['message'].'.';
                if (isset($response['description'])) {
                    $message .= ' '.$response['description'];
                }
            }
            throw new NotFoundException($message);
        }

        $ord2->updateOrderInfo('price_in_btc', [
            'baseName' => price.' (BTC)',
            'value' => $price
        ]);

        $ord2->updateOrderInfo('address', [
            'baseName' => 'Address',
            'value' => $response['address']
        ]);

        $ord2->updateOrderInfo('confirmations', [
            'baseName' => Confirmations.' (BTC)',
            'value' => 0
        ]);

        $script = 'document.body.insertAdjacentHTML("beforeend", `<div id="pgc_bit" class="pgdiv phidden"><div class="tddiv"><a id="ac_bit" class="ascroll" data-path="/ac_bit">&nbsp;</a><div id="vbit" class="virtuals"><div id="bbit" class="vdis"><div id="c_bit" class="droppable" style="width:320px;min-height:350px;text-align:center;padding:60px 20px;background-color:white;line-height:30px">'.pleaseSend.' '.$price.' BTC <span style="font-size:12px;">'.$response['address'].'</span> <img style="margin:15px" src="https://chart.googleapis.com/chart?chs=250x250&cht=qr&chl='.urlencode('bitcoin:'.$response['address'].'?amount='.$price).'" alt=""></div></div></div></div></div>`);';
        $script .= 'popupOpen("c_bit");';
        $ord2->addScript($script);

        return '';
    }

    public function settings($p)
    {
        if (!isset($p['settings']['bitcoinAddress']))
        {
            $p['settings']['bitcoinAddress'] = '';
        }

        if (!isset($p['settings']['secret']))
        {
            $p['settings']['secret'] = substr(md5(rand(0, 99999).' '.time()), rand(0, 4), 16);
        }

        if (!isset($p['settings']['confirmations']))
        {
            $p['settings']['confirmations'] = '3';
        }

        if (!isset($p['settings']['apikey']))
        {
            $p['settings']['apikey'] = '';
        }

        return $p;
    }

    public function update($data)
    {
        return $data;
    }
}