<?php
use Shop\Entity\Order;
use Shop\Usecase\GetOrderByIdUsecase;
use Shop\Usecase\RunEventAndSaveOrderUsecase;
use Shop\Exception\NotFoundException;

class paymentFastSpring_settings_mapper
{
    private $createOrderUrl = 'https://sites.fastspring.com/';

    public function success()
    {
        header('Content-Type: text/html; charset=utf-8');

        if (!isset($_REQUEST['security_data']) || !isset($_REQUEST['security_hash']))
        {
            throw new NotFoundException('Error data');
        }

        $order_id = 0;
        if(!empty($_REQUEST['OrderReferrer'])) $order_id = (int) $_REQUEST['OrderReferrer'];
        if (!$order_id) {
            throw new NotFoundException('Order Id error.');
        }

        $ordUsecase = new GetOrderByIdUsecase();
        $ord2 = $ordUsecase->run($order_id);

        if ($ord2->event() !== 'submit' && $ord2->event() !== 'work' && $ord2->event() !== 'cancel')
        {
            return;
        }

        if(md5($_REQUEST['security_data'].$ord2->payment()->settings('secretKey')) != $_REQUEST['security_hash'])
        {
            throw new NotFoundException('Security check failed.');
        }

        if (!$ord2->payment()->id() || $ord2->payment()->status() == 'off') {
            throw new NotFoundException('Error');
        }

        includeLangConst($ord2->langId());

        $orderEvent = new RunEventAndSaveOrderUsecase($ord2);
        $orderEvent->run('paid');
    }

    public function getPaymentUrl(Order $ord2)
    {
        $IDN = new \idna_convert(array('idn_version'=>2008));

        return $this->createOrderUrl.$ord2->payment()->settings('companyId').'/instant/'.str_replace(' ', '', $ord2->name()).
               '?operation=create&destination=contents&contact_fname= &contact_lname= &contact_email='.($ord2->customer()->email() ? urlencode($IDN->encode($ord2->customer()->email())) : '').
               '&referrer='.$ord2->id().
               ($ord2->payment()->status() == 'test' ? '&mode=1' : '').
               ($ord2->price() ? '&tags='.urlencode('total='.(100 * $ord2->price())) : '');

    }

    public function settings($p)
    {
        if (!isset($p['settings']['companyId']))
        {
            $p['settings']['companyId'] = '';
        }

        if (!isset($p['settings']['secretKey']))
        {
            $p['settings']['secretKey'] = '';
        }

        return $p;
    }

    public function update($data)
    {
        return $data;
    }
}