<?php
use Shop\Entity\Order;
use Shop\Usecase\GetOrderByIdUsecase;
use Shop\Usecase\RunEventAndSaveOrderUsecase;
use Shop\Exception\NotFoundException;

class paymentOnpay_settings_mapper
{
    private $createOrderUrl = 'https://secure.onpay.ru/pay/';
    public $paymentType = array(''=>'',
                                'ACL'=>'[ACL] Alfa Click RUR',
                                'BBR'=>'[BBR] Fast bank payments',
                                'BL2'=>'[BL2] Pay with Beeline',
                                'BL3'=>'[BL3] Pay with Beeline',
                                'BTC'=>'[BTC] milliBitcoin',
                                'BTR'=>'[BTR] Bank transfer RUR',
                                'SBR'=>'[SBR] Sberbank-Online',
                                'CNT'=>'[CNT] Contact',
                                'EUR'=>'[EUR] Euro account',
                                'FSG'=>'[FSG] Internet banking and ATMs',
                                'SVB'=>'[SVB] Svyaznoy bank, RUR',
                                'URB'=>'[URB] Uralsib bank',
                                'MDM'=>'[MDM] MDM Bank',
                                'HBK'=>'[HBK] Pay terminal Elexnet (Handybank)',
                                'MKB'=>'[MKB] MKB',
                                'JKH'=>'[JKH] Housing fee, RUR',
                                'LIQ'=>'[LIQ] Visa MasterCard RUR (LiqPay)',
                                'MG2'=>'[MG2] Pay with Megafon',
                                'MOB'=>'[MOB] Mobile payments',
                                'MT2'=>'[MT2] Pay with MTS',
                                'OCE'=>'[OCE] Visa MasterCard RUR N2',
                                'RAP'=>'[RAP] Rapida',
                                'MTC'=>'[MTC] MTS, RUR',
                                'SVZ'=>'[SVZ] Svyaznoy, RUR',
                                'EUS'=>'[EUS] Euroset, RUR',
                                'RSB'=>'[RSB] Russian Standard',
                                'SVM'=>'[SVM] Svyaznoy-bank',
                                'SVN'=>'[SVN] Svyaznoy-bank',
                                'TCR'=>'[TCR] Visa MasterCard RUR (TCR)',
                                'TE2'=>'[TE2] Mobile payments',
                                'TL2'=>'[TL2] Mobile payments',
                                'UNR'=>'[UNR] Visa/Mastercard /JCB/DC N1',
                                'USD'=>'[USD] Wire Transfer USD',
                                'UT3'=>'[UT3] Utel',
                                'MG3'=>'[MG3] Megafon',
                                'SM3'=>'[SM3] Smarts',
                                'TL3'=>'[TL3] TELE2',
                                'MT3'=>'[MT3] MTS',
                                'UTL'=>'[UTL] Rostelecom-Ural (Utel)',
                                'VTB'=>'[VTB] VTB',
                                'WMZ'=>'[WMZ] Webmoney WMZ',
                                'WMB'=>'[WMB] Webmoney WMB',
                                'WME'=>'[WME] Webmoney WME',
                                'WMR'=>'[WMR] Webmoney WMR',
                                'WMU'=>'[WMU] Webmoney WMU',
                                'UAT'=>'[UAT] Terminals in Ukraine',
                                'YDX'=>'[YDX] Yoomoney'
    );

    public function success()
    {
        header("Content-type: application/json; charset=utf-8");

        $_REQUEST = json_decode(file_get_contents('php://input'), true);

        if (!isset($_REQUEST['type']) ||
            !isset($_REQUEST['pay_for']) ||
            !isset($_REQUEST['signature']))
        {
            throw new NotFoundException('bad');
        }

        $orderId = (int) $_REQUEST['pay_for'];

        $ordUsecase = new GetOrderByIdUsecase();
        $ord2 = $ordUsecase->run($orderId);
        $secretKey = $ord2->payment()->settings('secretKey');

        if ($ord2->event() !== 'submit' && $ord2->event() !== 'work' && $ord2->event() !== 'cancel')
        {
            print json_encode(array('status' => true, 'pay_for' => $_REQUEST['pay_for'], "signature" => sha1('check;true;'.$_REQUEST['pay_for'].';'.$secretKey)));
            return;
        }

        if (!$ord2->payment()->id() || $ord2->payment()->status() == 'off')
        {
            throw new NotFoundException('bad');
        }

        includeLangConst($ord2->langId());


        if ($_REQUEST['type'] == 'check') {

            if (sha1('check;'.$_REQUEST['pay_for'].';'.$this->to_float($_REQUEST['amount']).';'.$_REQUEST['way'].';'.$_REQUEST['mode'].';'.$secretKey) != $_REQUEST['signature'] || 'fix' != $_REQUEST['mode'])
            {
                throw new NotFoundException('bad');
            }
            print json_encode(array('status' => true, 'pay_for' => $_REQUEST['pay_for'], "signature" => sha1('check;true;'.$_REQUEST['pay_for'].';'.$secretKey)));
            return;

        } elseif ($_REQUEST['type'] == 'pay') {

        	if (sha1('pay;'.$_REQUEST['pay_for'].';'.$this->to_float($_REQUEST['payment']['amount']).';'.$_REQUEST['payment']['way'].';'.$this->to_float($_REQUEST['balance']['amount']).';'.$_REQUEST['balance']['way'].';'.$secretKey) == $_REQUEST['signature'])
            {
                $orderEvent = new RunEventAndSaveOrderUsecase($ord2);
                $orderEvent->run('paid');

	            print json_encode(array('status' => true, 'pay_for' => $_REQUEST['pay_for'], "signature" => sha1('check;true;'.$_REQUEST['pay_for'].';'.$secretKey)));
                return;
            }
        }

        throw new NotFoundException('bad');
    }

    public function to_float($sum) {
	    $sum2 = round(floatval($sum), 2);
	    $sum = sprintf("%01.2f", $sum2);
	    if (substr($sum, -1) == "0") {
	        $sum = sprintf("%01.1f", $sum2);
	    }
	    return $sum;
	}

    public function getPaymentUrl(Order $ord2)
    {
    	if ($ord2->customer()->email()) {
            $IDN = new \idna_convert(array('idn_version'=>2008));
            $userEmail = $IDN->encode($ord2->customer()->email());
        }
        $price = $this->to_float($ord2->price());

        return $this->createOrderUrl.$ord2->payment()->settings('login').
               '?price='.urlencode($price).
               '&ticker='.($ord2->payment()->status() == 'test' ? 'TST' : $ord2->payment()->currency()).
               '&pay_mode=fix'.
               ($ord2->payment()->method()? '&one_way='.urlencode($ord2->payment()->method()) : '').
               '&md5='.urlencode(md5(
                    'fix;'.$price.';'.
                    ($ord2->payment()->status() == 'test' ? 'TST' : $ord2->payment()->currency()).';'.
                    $ord2->id().';yes;'.$ord2->payment()->settings('secretKey')
                )).
               '&pay_for='.$ord2->id().
               '&note='.urlencode($ord2->name().' #'.$ord2->id()).
               '&convert=yes'.
               '&ln='.(CUR_LANG == 'ru' || CUR_LANG == 'uk' || CUR_LANG == 'kk' ? 'ru' : 'en').
               ($ord2->payment()->settings('successURL') ? '&url_success='.urlencode($ord2->payment()->settings('successURL')) : '').
               ($ord2->payment()->settings('priceFinal') ? '&price_final='.urlencode($ord2->payment()->settings('priceFinal')) : '').
               ($userEmail ? '&user_email='.urlencode($userEmail) : '');

    }

    public function settings($p)
    {
        if (!isset($p['settings']['login']))
        {
            $p['settings']['login'] = '';
        }

        if (!isset($p['settings']['secretKey']))
        {
            $p['settings']['secretKey'] = '';
        }

        if (!isset($p['settings']['successURL']))
        {
            $p['settings']['successURL'] = '';
        }

        if (!isset($p['settings']['priceFinal']))
        {
            $p['settings']['priceFinal'] = '';
        }

        return $p;
    }

    public function update($data)
    {
        return $data;
    }
}