<?php
use Shop\Entity\Order;
use Shop\Usecase\GetOrderByIdUsecase;
use Shop\Usecase\RunEventAndSaveOrderUsecase;
use Shop\Exception\NotFoundException;

class paymentPaypal_settings_mapper
{
    private $createOrderUrl = 'https://www.paypal.com/cgi-bin/webscr';
    public $paymentType = array('echeck'=>'This payment was funded with an eCheck', 'instant'=>'This payment was funded with PayPal balance, credit card, or Instant Transfer');

    public function success()
    {
        if (empty($_POST) || empty($_POST['invoice']))
        {
            throw new NotFoundException('bad');
        }

        $orderId = (int) substr($_POST['invoice'], 4);

        $ordUsecase = new GetOrderByIdUsecase();
        $ord2 = $ordUsecase->run($orderId);

        if ($ord2->event() !== 'submit' && $ord2->event() !== 'work' && $ord2->event() !== 'cancel')
        {
            print 'OK';
            return;
        }

        if (!$ord2->payment()->id() ||
            $ord2->payment()->status() == 'off' ||
            $ord2->price() <= 0)
        {
            throw new NotFoundException('bad');
        }

        includeLangConst($ord2->langId());

        if($ord2->payment()->status() == 'test') $url = "www.sandbox.paypal.com";
		else $url = "www.paypal.com";

        $postdata = "";
        foreach ($_POST as $key=>$value) $postdata .= $key.'='.urlencode($value).'&';
        $postdata .= "cmd=_notify-validate";

        $header = '';
        $header .= "POST /cgi-bin/webscr HTTP/1.1\r\n";
        $header .= "Host: ".$url."\r\n";
        $header .= "Content-Type: application/x-www-form-urlencoded\r\n";
        $header .= "Content-Length: " . strlen($postdata) . "\r\n";
        $header .= "Connection: close\r\n\r\n";
        $response = '';

        $fp = @fsockopen('ssl://'.$url, 443, $errno, $errstr, 30);
        if ($fp)
        {
	        @fputs($fp, $header.$postdata);
	        while (!@feof($fp)) {
	           $response = @fgets($fp, 1024);
	           $response = trim($response);
	           if (strcmp ($response, "VERIFIED") == 0) {
	              break;
	           } elseif (strcmp ($response, "INVALID") == 0) {
	              break;
	           }
	        }
	        @fclose($fp);
        }

		if ($response != "VERIFIED")
        {
            throw new NotFoundException('Could not verify transaction');
		}
		if($_POST["payment_status"] != "Completed" )
        {
            throw new NotFoundException('Incorrect status '.$_POST["payment_status"].$_POST["pending_reason"]);
		}
        $IDN = new \idna_convert(array('idn_version'=>2008));
		if ($_POST['receiver_email'] != $IDN->encode($ord2->payment()->settings('email')))
        {
            throw new NotFoundException('Incorrect merchant email');
		}
        if ($_POST['txn_type'] != 'cart')
        {
            throw new NotFoundException('Incorrect txn_type');
        }
        if($_POST["mc_gross"] != $ord2->price())
        {
            throw new NotFoundException('Incorrect total price ('.$ord2->price().' <> '.$_POST["mc_gross"].')');
        }
        if ($ord2->payment()->currency() != '' && $_POST['mc_currency'] != $ord2->payment()->currency())
        {
            throw new NotFoundException('Incorrect currency ('.$ord2->payment()->currency().' <> '.$_POST["mc_currency"].')');
        }

        $orderEvent = new RunEventAndSaveOrderUsecase($ord2);
        $orderEvent->run('paid');

        print 'OK';
    }

    public function getPaymentUrl(Order $ord2)
    {
    	if($ord2->payment()->status() == 'test') $this->createOrderUrl = 'https://www.sandbox.paypal.com/cgi-bin/webscr';
    	$lang = '';
        if (in_array(strtoupper(CUR_LANG), array('AU','AT','BE','BR','CA','CH','CN','DE','ES','GB','FR','IT','NL','PL','PT','RU','US'))) {
        	$lang = strtoupper(CUR_LANG);
        } elseif (CUR_LANG == 'uk') {
        	$lang = 'ru_RU';
        } elseif (CUR_LANG == 'tr') {
            $lang = 'tr_TR';
        }

        $IDN = new \idna_convert(array('idn_version'=>2008));

        return $this->createOrderUrl.
               '?business='.urlencode($IDN->encode($ord2->payment()->settings('email'))).
               '&amount_1='.urlencode($ord2->price()).
               '&item_name_1='.urlencode($ord2->name()).
               '&item_number_1='.urlencode(substr(DOMAIN,0,3).'-'.$ord2->id()).
               '&rm=0&upload=1&cmd=_cart&charset='.urlencode('utf-8').
               ($lang ? '&lc='.urlencode($lang) : '').
               ($ord2->payment()->currency() ? '&currency_code='.urlencode($ord2->payment()->currency()) : '').
               ($ord2->payment()->settings('successURL') ? '&return='.urlencode($ord2->payment()->settings('successURL')) : '').
               '&invoice='.urlencode(substr(DOMAIN,0,3).'-'.$ord2->id()).
               ($ord2->customer()->email() ? '&email='.urlencode($IDN->encode($ord2->customer()->email())) : '').
               '&notify_url='.urlencode(siteProtocol.'://'.DOMAIN.BASE_DIR.'/payment/Paypal/callback.php');
    }

    public function settings($p)
    {
        if (!isset($p['settings']['email']))
        {
            $p['settings']['email'] = '';
        }

        if (!isset($p['settings']['successURL']))
        {
            $p['settings']['successURL'] = '';
        }

        return $p;
    }

    public function update($data)
    {
        return $data;
    }
}