<?php
use Shop\Entity\Order;
use Shop\Usecase\GetOrderByIdUsecase;
use Shop\Usecase\RunEventAndSaveOrderUsecase;
use Shop\Exception\NotFoundException;

class paymentRobokassa_settings_mapper
{
	private $createOrderUrl = 'https://auth.robokassa.ru/Merchant/Index.aspx';

    public function success()
    {
        header('Content-Type: text/html; charset=utf-8');

		if (empty($_REQUEST['OutSum']) || empty($_REQUEST['InvId']) || empty($_REQUEST['SignatureValue']))
        {
            throw new NotFoundException('bad');
		}

        $orderId = (int) $_REQUEST['InvId'];

        $ordUsecase = new GetOrderByIdUsecase();
        $ord2 = $ordUsecase->run($orderId);

        if ($ord2->event() !== 'submit' && $ord2->event() !== 'work' && $ord2->event() !== 'cancel')
        {
            print 'OK'.$ord2->id()."\n";
            return;
        }

        if (!$ord2->payment()->id() ||
            $ord2->payment()->status() == 'off' ||
            $ord2->price() != $_REQUEST['OutSum'])
        {
            throw new NotFoundException('bad');
		}

        includeLangConst($ord2->langId());

		if (strtoupper(md5($_REQUEST['OutSum'].':'.$ord2->id().':'.$ord2->payment()->settings('pass2'))) == strtoupper($_REQUEST['SignatureValue']))
        {
            $orderEvent = new RunEventAndSaveOrderUsecase($ord2);
            $orderEvent->run('paid');

            print 'OK'.$ord2->id()."\n";
            return;
		}

        print 'bad';
    }

    public function getPaymentUrl(Order $ord2)
    {
		if ($ord2->customer()->email()) {
            $IDN = new \idna_convert(array('idn_version'=>2008));
            $userEmail = $IDN->encode($ord2->customer()->email());
        }

        if ($ord2->payment()->settings('sendCheck')) {
            $receipt = array('items' => array());
            if ($ord2->payment()->settings('sno'))
            {
                $receipt['sno'] = $ord2->payment()->settings('sno');
            }
            $receipt['items'][0] = array(
                'quantity' => (float) $ord2->productQuantity(),
                'name' => substr($ord2->productName(), 0, 64),
                'tax' => $ord2->payment()->settings('tax'),
                'sum' => $ord2->price()
            );
            if ($ord2->payment()->settings('paymentMethodType')) {
                $receipt['items'][0]['payment_method'] = $ord2->payment()->settings('paymentMethodType');
            }
            if ($ord2->payment()->settings('paymentSubjectType')) {
                $receipt['items'][0]['payment_object'] = $ord2->payment()->settings('paymentSubjectType');
            }
        }

		return $this->createOrderUrl.
               '?MrchLogin='.urlencode($ord2->payment()->settings('login')).
               '&OutSum='.urlencode($ord2->price()).
               '&InvId='.urlencode($ord2->id()).
               '&Desc='.urlencode($ord2->name()).
               '&SignatureValue='.urlencode(
                    md5(
                        $ord2->payment()->settings('login').':'.
                        $ord2->price().':'.
                        $ord2->id().':'.
                        ($ord2->payment()->settings('sendCheck') ? json_encode($receipt).':' : '').
                        $ord2->payment()->settings('pass1')
                    )
                ).
               '&Email='.urlencode($userEmail).
               ($ord2->payment()->status() == 'test' ? '&IsTest=1' : '').
               ($ord2->payment()->settings('sendCheck') ? '&receipt='.urlencode(json_encode($receipt)) : '').
               '&Encoding=utf-8';

    }

    public function settings($p)
    {
        if (!isset($p['settings']['login']))
        {
            $p['settings']['login'] = 'testLogin';
        }

        if (!isset($p['settings']['pass1']))
        {
            $p['settings']['pass1'] = 'test1pass';
        }

        if (!isset($p['settings']['pass2']))
        {
            $p['settings']['pass2'] = 'test2pass';
        }

        if (!isset($p['settings']['tax']))
        {
            $p['settings']['tax'] = '';
        }

        if (!isset($p['settings']['paymentSubjectType']))
        {
            $p['settings']['paymentSubjectType'] = '';
        }

        if (!isset($p['settings']['paymentMethodType']))
        {
            $p['settings']['paymentMethodType'] = '';
        }

        if (!isset($p['settings']['sno']))
        {
            $p['settings']['sno'] = '';
        }

        return $p;
    }

    public function update($data)
    {
        return $data;
    }
}