<?php
use Shop\Entity\Order;
use Shop\Usecase\GetOrderByIdUsecase;
use Shop\Usecase\RunEventAndSaveOrderUsecase;
use Shop\Exception\NotFoundException;

class paymentUnitpay_settings_mapper {
    public $paymentType = array(''=>'','card'=>'VISA/MasterCard/Maestro', 'applepay'=>'Apple Pay', 'samsungpay'=>'Samsung Pay', 'googlepay'=>'Google Pay', 'alfaClick'=>'Alfa Click', 'yandex'=>'Yoomoney', 'paypal'=>'PayPal', 'webmoney'=>'WMZ', 'webmoneyWmr'=>'WMR', 'mc'=>'Mobile');

    public function success()
    {
        header('Content-Type: text/html; charset=utf-8');

        try
        {
            if (!isset($_GET['params']['orderSum']) ||
                !isset($_GET['params']['orderCurrency']) ||
                !isset($_GET['params']['account']) ||
                !isset($_GET['method']))
            {
                throw new InvalidArgumentException('Order validation Error!');
            }

            $orderId = (int) $_GET['params']['account'];

            $ordUsecase = new GetOrderByIdUsecase();
            $ord2 = $ordUsecase->run($orderId);

            $unitPay = new UnitPay($ord2->payment()->settings('sendCheck') ? 'unitpay.ru' : 'unitpay.money', $ord2->payment()->settings('secretKey'));
            $unitPay->checkHandlerRequest();

            if ($ord2->event() !== 'submit' && $ord2->event() !== 'work' && $ord2->event() !== 'cancel')
            {
                echo $unitPay->getSuccessHandlerResponse('Pay Success');
                return;
            }

            if (!$ord2->payment()->id() ||
                $ord2->payment()->status() == 'off' ||
                $_GET['params']['orderCurrency'] != $ord2->payment()->currency() ||
                $ord2->price() != $_GET['params']['orderSum'])
            {
                throw new InvalidArgumentException('Order validation Error!');
            }

            includeLangConst($ord2->langId());

            switch ($_GET['method']) {
                case 'check':

                    echo $unitPay->getSuccessHandlerResponse('Check Success. Ready to pay.');
                    break;

                case 'pay':

                    $orderEvent = new RunEventAndSaveOrderUsecase($ord2);
                    $orderEvent->run('paid');

                    echo $unitPay->getSuccessHandlerResponse('Pay Success');
                    break;

            case 'error':

                echo $unitPay->getSuccessHandlerResponse('Error');
                break;

            }

        } catch (Exception $e) {

            echo $unitPay->getErrorHandlerResponse($e->getMessage());

        }
    }

    public function getPaymentUrl(Order $ord2)
    {
        try
        {
            if (!$ord2->payment()->method()) {
                return $this->form($ord2);
            }

            $unitPay = new UnitPay($ord2->payment()->settings('sendCheck') ? 'unitpay.ru' : 'unitpay.money', $ord2->payment()->settings('secretKey'));

            $params = array('account'     => $ord2->id(),
                            'currency'    => $ord2->payment()->currency(),
                            'desc'        => $ord2->name(),
                            'sum'         => $ord2->price());

            $params['signature'] = $unitPay->getSignature($params);
            $params['projectId'] = $ord2->payment()->settings('projectId');
            $params['paymentType'] = $ord2->payment()->method();

            if ($ord2->payment()->settings('successURL'))
            {
                $params['resultUrl'] = $ord2->payment()->settings('successURL');
            }
            if (CUR_LANG == 'ru' || CUR_LANG == 'uk' || CUR_LANG == 'kk')
            {
                $params['locale'] = 'ru';
            }
            if ($ord2->customer()->email())
            {
                $unitPay->setCustomerEmail($ord2->customer()->email());
            }
            if ($ord2->customer()->phone())
            {
                $unitPay->setCustomerPhone($ord2->customer()->phone());
            }

            if ($ord2->payment()->settings('sendCheck'))
            {
                $unitPay->setCashItems(array(
                    new CashItem(
                                    $ord2->productName(),
                                    $ord2->productQuantity(),
                                    round($ord2->price() / $ord2->productQuantity(), 2),
                                    $ord2->payment()->settings('paymentSubjectType')
                                )
                ));
            }

            $response = $unitPay->api('initPayment', $params);

            if (isset($response->result->type) && $response->result->type == 'redirect')
            {
                return $response->result->redirectUrl;
            }
            elseif (isset($response->result->type) && $response->result->type == 'invoice')
            {
                return $response->result->receiptUrl;
            }
            elseif (isset($response->error->message))
            {
                throw new NotFoundException('Error: '.$response->error->message);
            }

            throw new NotFoundException('Pay url empty');
        }
        catch (\Throwable $e)
        {
            throw new NotFoundException($e->getMessage());
        }
    }

    public function form(Order $ord2) {

        $unitPay = new UnitPay($ord2->payment()->settings('sendCheck') ? 'unitpay.ru' : 'unitpay.money', $ord2->payment()->settings('secretKey'));

        if ($ord2->customer()->email()) {
            $unitPay->setCustomerEmail($ord2->customer()->email());
        }
        if ($ord2->customer()->phone()) {
            $unitPay->setCustomerPhone($ord2->customer()->phone());
        }
        if ($ord2->payment()->settings('sendCheck'))
        {
            $unitPay->setCashItems(array(
                new CashItem(
                                $ord2->productName(),
                                $ord2->productQuantity(),
                                round($ord2->price() / $ord2->productQuantity(), 2),
                                $ord2->payment()->settings('paymentSubjectType')
                            )
            ));
        }

        $redirectUrl = $unitPay->form(
            $ord2->payment()->settings('projectId'),
            $ord2->price(),
            $ord2->id(),
            $ord2->name(),
            $ord2->payment()->currency()
        );

        return $redirectUrl;
    }

    public function settings($p)
    {
        if (!isset($p['settings']['projectId']))
        {
            $p['settings']['projectId'] = '';
        }

        if (!isset($p['settings']['secretKey']))
        {
            $p['settings']['secretKey'] = '';
        }

        if (!isset($p['settings']['successURL']))
        {
            $p['settings']['successURL'] = '';
        }

        if (!isset($p['settings']['paymentSubjectType']))
        {
            $p['settings']['paymentSubjectType'] = '';
        }

        return $p;
    }

    public function update($data)
    {
        return $data;
    }
}