<?php
use Shop\Entity\Order;
use Shop\Usecase\GetOrderByIdUsecase;
use Shop\Usecase\RunEventAndSaveOrderUsecase;
use Shop\Exception\NotFoundException;

class paymentWalletone_settings_mapper
{
    private $createOrderUrl = 'https://wl.walletone.com/checkout/checkout/Index';

    public function success()
    {
        header('Content-Type: text/html; charset=utf-8');

        if (!isset($_POST['WMI_MERCHANT_ID']) ||
            !isset($_POST['WMI_PAYMENT_AMOUNT']) ||
            !isset($_POST['WMI_PAYMENT_NO']) ||
            !isset($_POST['WMI_ORDER_STATE']) ||
            !isset($_POST['WMI_SIGNATURE']) ||
			'Accepted' != $_POST['WMI_ORDER_STATE'])
        {
			print "WMI_RESULT=RETRY&WMI_DESCRIPTION=bad";
            return;
        }

        $orderId = (int) $_REQUEST['WMI_PAYMENT_NO'];

        $ordUsecase = new GetOrderByIdUsecase();
        $ord2 = $ordUsecase->run($orderId);

        if ($ord2->event() !== 'submit' && $ord2->event() !== 'work' && $ord2->event() !== 'cancel')
        {
            print "WMI_RESULT=OK&WMI_DESCRIPTION=#".(int)$_POST['WMI_PAYMENT_NO'];
            return;
        }

        if (!$ord2->payment()->id() ||
            $ord2->payment()->status() == 'off' ||
            number_format($ord2->price(), 2, '.', '') != $_POST['WMI_PAYMENT_AMOUNT'])
        {
            print "WMI_RESULT=RETRY&WMI_DESCRIPTION=bad";
            return;
        }

        includeLangConst($ord2->langId());

		$values = "";
        uksort($_POST, "strcasecmp");
        foreach($_POST as $name => $value)
        {
            if ($name !== "WMI_SIGNATURE") $values .= $value;
        }
        $signature = base64_encode(pack("H*", md5($values . $ord2->payment()->settings('secretKey'))));

        if ($signature == $_POST["WMI_SIGNATURE"])
        {
            $orderEvent = new RunEventAndSaveOrderUsecase($ord2);
            $orderEvent->run('paid');

            print "WMI_RESULT=OK&WMI_DESCRIPTION=#".(int)$_POST['WMI_PAYMENT_NO'];
            return;
        }

        print "WMI_RESULT=RETRY&WMI_DESCRIPTION=bad";
    }

    public function getPaymentUrl(Order $ord2)
    {
		$dataSet = array('WMI_MERCHANT_ID' => $ord2->payment()->settings('accountId'),
                         'WMI_CURRENCY_ID' => $ord2->payment()->currency(),
                         'WMI_PAYMENT_AMOUNT' => number_format($ord2->price(), 2, '.', ''),
                         'WMI_PAYMENT_NO' => $ord2->id(),
                         'WMI_DESCRIPTION' => $ord2->name());

		if ($ord2->customer()->email())
        {
			$IDN = new \idna_convert(array('idn_version'=>2008));
			$dataSet['WMI_CUSTOMER_EMAIL'] = $IDN->encode($ord2->customer()->email());
		}
		if ($ord2->payment()->settings('successURL'))
        {
            $dataSet['WMI_SUCCESS_URL'] = $ord2->payment()->settings('successURL');
        }

		$fieldValues = "";
        uksort($dataSet, "strcasecmp");
        foreach($dataSet as $value) {
			$value = iconv("utf-8", "windows-1251", $value);
            $fieldValues .= $value;
		}
		$sign = base64_encode(pack("H*", md5($fieldValues . $ord2->payment()->settings('secretKey'))));

        $dataSet['WMI_SIGNATURE'] = $sign;

        try
        {
            $ch = curl_init($this->createOrderUrl);
            curl_setopt($ch, CURLOPT_TIMEOUT, 30);
            curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 5);
            curl_setopt($ch, CURLOPT_HEADER, 1);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_POST, true);
            curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($dataSet, '', '&'));
            $res = curl_exec($ch);
            $error = curl_errno($ch);
            $info = curl_getinfo($ch);

            if ($error)
            {
                throw new NotFoundException($error.': '.curl_error($ch));
            }

            if (!$info['redirect_url'])
            {
                $res = explode("<html>", $res);
                throw new NotFoundException(strip_tags($res[1]));
            }
            curl_close($ch);

            return $info['redirect_url'];
        }
        catch (\Throwable $e)
        {
            throw new NotFoundException($e->getMessage());
        }


    }

    public function settings($p)
    {
        if (!isset($p['settings']['accountId']))
        {
            $p['settings']['accountId'] = '';
        }

        if (!isset($p['settings']['secretKey']))
        {
            $p['settings']['secretKey'] = '';
        }

        if (!isset($p['settings']['successURL']))
        {
            $p['settings']['successURL'] = '';
        }

        return $p;
    }

    public function update($data)
    {
        return $data;
    }
}