<?php
use Common\Lib\DB;

class update_settings_mapper
{
    const TIMEOUT = 10;
    private $download_stream = null;
    private $file_length = 0;
    private $file_md5 = null;
    private $current_content_length = 0;

    public function getLastVersion($isCash = true)
    {
        $config = main::mapper('config_settings')->getData();
        if (!$isCash || $config['nversionTime'] < mktime(date("H") - 1, 0, 0, date("m"), date("d"), date("Y")))
        {
            $url = 'http://cportfolio.com/version.txt';
            $version = cp_get_contents($url);
            if ($version && (int)$version[0] == 0) $version = false;
            if ($version)
            {
                main::mapper('config_settings')->setVersion($version);
            }
            else
            {
                main::addError(connectionError);
            }

        }

    }

    public function check()
    {
        $config = main::mapper('config_settings')->getData();
        if ($config['nversion'] == $config['cversion'] && $config['nversionTime'] < mktime(date("H"), 0, 0, date("m"), date("d")-1, date("Y")))
        {
            $url = 'http://cportfolio.com/version.txt';
            $version = cp_get_contents($url);
            if ($version && (int)$version[0] == 0) $version = false;
            if ($version)
            {
                main::mapper('config_settings')->setVersion($version);
            }
        }
    }

    public function update($data)
    {
        $config = main::mapper('config_settings')->getData();

        $url = 'http://cportfolio.com/update-path.php?type=json&lang='.CUR_LANG.'&cversion='.urlencode($config['cversion']).'&DOMAIN='.DOMAIN;
        $return = cp_get_contents($url);
        if ($return)
        {
            $return = json_decode($return, true);
        }

        if (isset($return['isError']))
        {
        	main::addError(byLicenseKey2.'<br>'.$return['error']);
        	return false;
        }

        if (empty($return['path']))
        {
        	main::addError(downloadError);
        	return false;
        }

		set_time_limit(10*60);
		ini_set('max_execution_time', 10*60);
		ini_set("memory_limit", '-1');

        if ($this->download($return['path'], ROOT_PATH.'cms.zip'))
        {
            if (file_exists(ROOT_PATH.'cms.zip'))
            {
	            $arc = new PclZip(ROOT_PATH.'cms.zip');
	            $list = $arc->extract(PCLZIP_OPT_REPLACE_NEWER,
	                                  PCLZIP_OPT_PATH, ROOT_PATH);
                unlink(ROOT_PATH.'cms.zip');
	            if ($list == 0) {
	            	main::addError($arc->errorInfo(true));
	            	return false;
                }
                if (function_exists('opcache_reset')) opcache_reset();
	        }

            if (file_exists(ROOT_PATH.'update.php'))
            {
                $version = include(ROOT_PATH.'update.php');
                main::mapper('config_settings')->setVersion('', $version);
                $this->getLastVersion(false);
            	unlink(ROOT_PATH.'update.php');
            }
    	}
    }

    private function download($fromPath, $toPath)
    {
    	$download = false;

    	$this->download_stream = fopen($toPath,'wb');

    	if(!$this->download_stream || !is_resource($this->download_stream)) {
    		main::addError(str_replace('%0.var.value%', realpath($toPath), isWrite_core));
    		if (is_resource($this->download_stream)) {
    			fclose($this->download_stream);
    			unlink($toPath);
    		}
    		return false;
    	}

        if(extension_loaded('curl') && function_exists('curl_init')) {
            $download = $this->downloadCurl($fromPath);
        } else {
            $download = $this->downloadFopen($fromPath);
        }

        fclose($this->download_stream);

        if ($download) {

        	if ($this->file_length) {
                $filesize = filesize($toPath);
        		if ($filesize && $this->file_length != $filesize) {
		            main::addError('(2) '.downloadError);
		            unlink($toPath);
		            return false;
        		}
	        }

	        if ($this->file_md5) {
	        	$md5 = md5_file($toPath);
	        	if ($md5 && (strcasecmp($this->file_md5, $md5) != 0)) {
		            main::addError('(3) '.downloadError);
		            unlink($toPath);
		            return false;
	        	}
	        }

        }

        if (!$download) unlink($toPath);
        return $download;
    }

    private function downloadCurl($filePath)
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_HEADER, 0);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_TIMEOUT, self::TIMEOUT*60);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, self::TIMEOUT);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch, CURLOPT_WRITEFUNCTION, array(&$this, 'curlWriteHandler'));
        curl_setopt($ch, CURLOPT_HEADERFUNCTION, array(&$this, 'curlHeaderHandler'));
        curl_setopt($ch, CURLOPT_URL, $filePath);

        $res = curl_exec($ch);
        if ($errno =  curl_errno($ch)) {
            main::addError('(4) '.downloadError.' - curl error: '.$errno.'#'.curl_error($ch));
	        curl_close($ch);
	        return false;
        }

        $status = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        if ($status != 200) {
        	curl_close($ch);
        	main::addError('(5) '.downloadError);
        	return false;
        }

        curl_close($ch);
        return true;
    }

    private function curlWriteHandler($ch, $chunk)
    {
        $size = fwrite($this->download_stream, $chunk);
        $this->current_content_length += $size;
        return $size;
    }

    private function curlHeaderHandler($ch, $header)
    {
        $header_matches = null;
        if (preg_match('/content-length:\s*(\d+)/i', $header, $header_matches)) {
            $this->file_length = intval($header_matches[1]);
        } elseif(preg_match('/content-md5:\s*([\da-f]{32})/i', $header, $header_matches)){
            $this->file_md5 = $header_matches[1];
        }
        return strlen($header);
    }

    private function downloadFopen($filePath)
    {
        $source_stream = null;

        $default_socket_timeout = ini_set('default_socket_timeout', self::TIMEOUT);
        $source_stream = fopen($filePath, 'r');
        ini_set('default_socket_timeout', $default_socket_timeout);

        if (!$source_stream) {
            main::addError('(6) '.downloadError);
	        fclose($source_stream);
	        return false;
        }

        $this->getStreamInfo($source_stream);

        $retry_counter = 0;
        while(($delta = stream_copy_to_stream($source_stream, $this->download_stream, 102400)) ||
              ($this->file_length && ($this->current_content_length<$this->file_length) && (++$retry_counter<20)) ||
              (!$this->file_length && (++$retry_counter<3))) {

            if ($delta) {
                $this->current_content_length += $delta;
                $retry_counter = 0;
            } else {
                sleep(3);
            }

        }

        fclose($source_stream);
        return true;

    }

    private function getStreamInfo($source_stream, $download_content_length = 4096)
    {
    	$stream_meta_data = stream_get_meta_data($source_stream);
        $buf = stream_get_contents($source_stream, $download_content_length);

        $this->current_content_length = min($download_content_length, strlen($buf));

        $stream_seekable = isset($stream_meta_data['seekable']) ? $stream_meta_data['seekable'] : false;

        $headers = array();
        if (isset($stream_meta_data["wrapper_data"]["headers"])) {
            $headers = $stream_meta_data["wrapper_data"]["headers"];
        } elseif(isset($stream_meta_data["wrapper_data"])) {
            $headers = $stream_meta_data["wrapper_data"];
        }

        $header_matches = null;
        foreach($headers as $header){
            if (preg_match('/content-length:\s*(\d+)/i',$header,$header_matches)) {
                $this->file_length=intval($header_matches[1]);
            } elseif (preg_match('/content-md5:\s*([\da-f]{32})/i',$header,$header_matches)) {
                $this->file_md5=$header_matches[1];
            }
        }


        if ($buf){
            fwrite($this->download_stream, $buf);
        }

    }
}