/* =========================================================
 * bootstrap-colorpicker.js 
 * http://www.eyecon.ro/bootstrap-colorpicker
 * =========================================================
 * Copyright 2012 Stefan Petre
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ========================================================= */
 
!function( $ ) {
	
	// Color object
	
	var Color = function(val) {
		this.value = {
			h: 1,
			s: 1,
			b: 1,
			a: 1
		};
		this.setColor(val);
	};
	
	Color.prototype = {
		constructor: Color,
		
		//parse a string to HSB
		setColor: function(val){
			val = val.toLowerCase();
			var that = this;
			$.each( CPGlobal.stringParsers, function( i, parser ) {
				var match = parser.re.exec( val ),
					values = match && parser.parse( match ),
					space = parser.space||'rgba';
				if ( values ) {
					that.value = CPGlobal.RGBtoHSB.apply(null, values);
					return false;
				}
			});
		},
		
		setHue: function(h) {
			this.value.h = 1- h;
		},
		
		setSaturation: function(s) {
			this.value.s = s;
		},
		
		setLightness: function(b) {
			this.value.b = 1- b;
		},
		
		setAlpha: function(a) {
			this.value.a = parseInt((1 - a)*100, 10)/100;
		},
		
		// HSBtoRGB from RaphaelJS
		// https://github.com/DmitryBaranovskiy/raphael/
		toRGBA: function(h, s, b, a) {
			if (!h) {
				h = this.value.h;
				s = this.value.s;
				b = this.value.b;
			}
			h *= 360;
			var R, G, B, X, C;
			h = (h % 360) / 60;
			C = b * s;
			X = C * (1 - Math.abs(h % 2 - 1));
			R = G = B = b - C;

			h = ~~h;
			R += [C, X, 0, 0, X, C][h];
			G += [X, C, C, X, 0, 0][h];
			B += [0, 0, X, C, C, X][h];
			return {
				r: Math.round(R*255),
				g: Math.round(G*255),
				b: Math.round(B*255),
				a: a||this.value.a
			};
		},

		RGBAstring: function(h, s, b, a) {
			var rgb = this.toRGBA(h, s, b, a);
			return 'rgba('+parseInt(rgb.r)+','+parseInt(rgb.g)+','+parseInt(rgb.b)+','+rgb.a+')';
		},

		toRGB: function(h, s, b, a) {
			var rgb = this.toRGBA(h, s, b, a);
			return 'rgb('+parseInt(rgb.r)+','+parseInt(rgb.g)+','+parseInt(rgb.b)+')';
		}


	};
	
	// Picker object
	
	var Colorpicker = function(element, options){
		this.element = $(element);
		var format = options.format||this.element.data('color-format')||'rgb';
		this.format = CPGlobal.translateFormats[format];
		this.picker = $(CPGlobal.template)
							.appendTo('body')
							.on('mousedown', 'div,span', $.proxy(this.mousedown, this))
							.on('input', 'input', $.proxy(this.keyup, this));

		;

		this.element.on({
			'click': $.proxy(this.show, this)
		});
		if (format == 'rgba') {
			this.picker.addClass('alpha');
			this.picker.find('.ok').addClass('ok40');
			this.alpha = this.picker.find('.colorpicker-alpha')[0].style;
		}
		
		this.preview = this.picker.find('.color-line')[0].style;
		this.base = this.picker.find('.colorpicker-saturation')[0].style;
		for (var i = 1; i <= 27; i++) {
			var data = this.element.data('c'+i)
			if(!data) continue;
			col = new Color(data);
			this.picker.find('#c'+i)[0].style.backgroundColor = col.toRGB();
		}

		this.update();
	};
	
	Colorpicker.prototype = {
		constructor: Colorpicker,
		show: function(e) {
			if (this.picker.css('display') != 'none') {
				$(".colorpicker").css('display', 'none');
				this.color = new Color(this.colorCancel);
				this.picker.find('i')
				.eq(0).css({left: this.color.value.s*100, top: 100 - this.color.value.b*100}).end()
				.eq(1).css('top', 100 * (1 - this.color.value.h)).end()
				.eq(2).css('top', 100 * (1 - this.color.value.a));
				this.previewColor();
				this.picker.hide();
			} else {
				$(".colorpicker").css('display', 'none');
				this.picker.show();
			}
			this.height = this.element.outerHeight();
			this.place();
			$(window).on('resize', $.proxy(this.place, this));
			if (e) {
				e.stopPropagation();
				e.preventDefault();
			}
			$('#dialog').on({
				'scroll': $.proxy(this.hide, this)
			});
			this.element.trigger({
				type: 'changeColor',
				color: this.color
			});
			this.element.trigger({
				type: 'show',
				color: this.color
			});
		},
		
		update: function(){
			this.colorCancel = this.element.data('color');
			this.color = new Color(this.element.data('color'));
			try {
				this.element.find('div div')[0].style.backgroundColor = this.format.call(this);
			} catch(e) {
				this.color.value.a = 1;
				this.element.find('div div')[0].style.backgroundColor = this.color.toRGB();
				this.element.find('input').prop('value', this.color.toRGB());
				if (this.element.attr("vs")) {
					if (this.element.attr("vs") == 'tbcolor') tbcolor = this.color.toRGB();
					else if (this.element.attr("vs") == 'ttcolor') ttcolor = this.color.toRGB();
					else if (this.element.attr("vs") == 'thcolor') thcolor = this.color.toRGB();
				}
			}
			this.picker.find('i')
				.eq(0).css({left: this.color.value.s*100, top: 100 - this.color.value.b*100}).end()
				.eq(1).css('top', 100 * (1 - this.color.value.h)).end()
				.eq(2).css('top', 100 * (1 - this.color.value.a));
			this.previewColor();
			this.picker.find('.infos').val(this.format.call(this));
		},

		update2: function(col){
			if (!col) return;
			this.colorCancel = col;
			this.color = new Color(col);
			this.element.trigger({
				type: 'changeColor',
				color: this.color
			});
			this.picker.find('i')
				.eq(0).css({left: this.color.value.s*100, top: 100 - this.color.value.b*100}).end()
				.eq(1).css('top', 100 * (1 - this.color.value.h)).end()
				.eq(2).css('top', 100 * (1 - this.color.value.a));
			this.previewColor();
			this.picker.find('.infos').val(this.format.call(this));
		},

		hide: function(){
			this.picker.hide();
			if (this.color.value.a == 1) {
				this.element.find('input').prop('value', this.color.toRGB());
				this.element.find('div div')[0].style.backgroundColor = this.color.toRGB();
			} else {
				try {
					this.element.find('div div')[0].style.backgroundColor = this.format.call(this);
					this.element.find('input').prop('value', this.format.call(this));
				} catch(e) {
					this.element.find('div div')[0].style.backgroundColor = this.color.toRGB();
					this.element.find('input').prop('value', this.color.toRGB());
				}
			}
			$(window).off('resize', this.place);
			this.element.trigger({
				type: 'hide',
				color: this.color
			});
		},

		ok: function(){
			this.picker.hide();
			$(document).off('mousedown', 'div,span', this.hide);
			if (this.color.value.a == 1) {
				this.element.find('input').prop('value', this.color.toRGB());
				this.element.find('div div')[0].style.backgroundColor = this.color.toRGB();
				this.colorCancel = this.color.toRGB();
			} else {
				try {
					this.element.find('div div')[0].style.backgroundColor = this.format.call(this);
					this.element.find('input').prop('value', this.format.call(this));
					this.colorCancel = this.format.call(this);
				} catch(e) {
					this.element.find('div div')[0].style.backgroundColor = this.color.toRGB();
					this.element.find('input').prop('value', this.color.toRGB());
					this.colorCancel = this.color.toRGB();
				}
			}
			var id = $("#"+this.element.attr("id").replace("color", "")+'Check').attr('id');
			if (id) $("#"+id).prop('checked', true);
			if (id == 'tbCheck') {
				changeColor(id);
			}

		},

		place: function(){
			var offset = this.element.offset();
			this.picker.css({
				top: offset.top + this.height,
				left: offset.left
			});
		},

		//preview color change
		previewColor: function(){
			try {
				this.preview.backgroundColor = this.format.call(this);
			} catch(e) {
				this.preview.backgroundColor = this.color.toRGB();
			}
			this.base.backgroundColor = this.color.toRGB(this.color.value.h, 1, 1, 1);
			
			if (this.alpha) {
				this.alpha.backgroundColor = this.color.toRGB();
			}

		},
		
		pointer: null,
		
		slider: null,
		
		mousedown: function(e){
			e.stopPropagation();
			e.preventDefault();
			
			var target = $(e.target);

			//detect the slider and set the limits and callbacks
			var zone = target.closest('div');
			if (!zone.is('.colorpicker') && !zone.is('.mousedown') && !zone.is('.color-line') && !zone.is('.sets') && !zone.is('.st')) {/*cportfolio*/
				if (zone.is('.colorpicker-saturation')) {
					this.slider = $.extend({}, CPGlobal.sliders['saturation']);
				} 
				else if (zone.is('.colorpicker-hue')) {
					this.slider = $.extend({}, CPGlobal.sliders['hue']);
				}
				else if (zone.is('.colorpicker-alpha')) {
					this.slider = $.extend({}, CPGlobal.sliders['alpha']);
				}
				var offset = zone.offset();
				//reference to knob's style
				this.slider.knob = zone.find('i')[0].style;
				this.slider.left = e.pageX - offset.left;
				this.slider.top = e.pageY - offset.top;
				this.pointer = {
					left: e.pageX,
					top: e.pageY
				};
				//trigger mousemove to move the knob to the current position
				$(document).on({
					mousemove: $.proxy(this.mousemove, this),
					mouseup: $.proxy(this.mouseup, this)
				}).trigger('mousemove');
			} else if (target.is('.ok') || target.is('.color-line')) {/*cportfolio*/
				this.ok();
			} else if (zone.is('.st')) {
				this.update2(this.element.data(target.closest('.st').attr("id")));
			}
			return false;
		},
		keyup: function(){
			var col = this.picker.find(".infos").val();
			if (!col) return;
			this.color = new Color(col);
			this.element.trigger({
				type: 'changeColor',
				color: this.color
			});
			this.previewColor();
		},
		mousemove: function(e){
			e.stopPropagation();
			e.preventDefault();
			var left = Math.max(
				0,
				Math.min(
					this.slider.maxLeft,
					this.slider.left + ((e.pageX||this.pointer.left) - this.pointer.left)
				)
			);
			var top = Math.max(
				0,
				Math.min(
					this.slider.maxTop,
					this.slider.top + ((e.pageY||this.pointer.top) - this.pointer.top)
				)
			);
			this.slider.knob.left = left + 'px';
			this.slider.knob.top = top + 'px';
			if (this.slider.callLeft) {
				this.color[this.slider.callLeft].call(this.color, left/100);
			}
			if (this.slider.callTop) {
				this.color[this.slider.callTop].call(this.color, top/100);
			}
			this.previewColor();
			this.picker.find('.infos').val(this.format.call(this));
			this.element.trigger({
				type: 'changeColor',
				color: this.color
			});
			return false;
		},
		
		mouseup: function(e){
			e.stopPropagation();
			e.preventDefault();
			$(document).off({
				mousemove: this.mousemove,
				mouseup: this.mouseup
			});
			return false;
		}
	}

	$.fn.colorpicker = function ( option ) {
		return this.each(function () {
			var $this = $(this),
				data = $this.data('colorpicker'),
				options = typeof option == 'object' && option;
			if (!data) {
				$this.data('colorpicker', (data = new Colorpicker(this, $.extend({}, $.fn.colorpicker.defaults,options))));
			}
			if (typeof option == 'string') data[option]();
		});
	};

	$.fn.colorpicker.defaults = {
	};
	
	$.fn.colorpicker.Constructor = Colorpicker;
	
	var CPGlobal = {
	
		// translate a format from Color object to a string
		translateFormats: {
			'rgb': function(){
				var rgb = this.color.toRGBA();
				return 'rgb('+rgb.r+','+rgb.g+','+rgb.b+')';
			},
			
			'rgba': function(){
				var rgb = this.color.toRGBA();
				return 'rgba('+rgb.r+','+rgb.g+','+rgb.b+','+rgb.a+')';
			}
			
		},
		
		sliders: {
			saturation: {
				maxLeft: 100,
				maxTop: 100,
				callLeft: 'setSaturation',
				callTop: 'setLightness'
			},
			
			hue: {
				maxLeft: 0,
				maxTop: 100,
				callLeft: false,
				callTop: 'setHue'
			},
			
			alpha: {
				maxLeft: 0,
				maxTop: 100,
				callLeft: false,
				callTop: 'setAlpha'
			}
		},
		
		// HSBtoRGB from RaphaelJS
		// https://github.com/DmitryBaranovskiy/raphael/
		RGBtoHSB: function (r, g, b, a){
			r /= 255;
			g /= 255;
			b /= 255;

			var H, S, V, C;
			V = Math.max(r, g, b);
			C = V - Math.min(r, g, b);
			H = (C == 0 ? null :
				 V == r ? (g - b) / C :
				 V == g ? (b - r) / C + 2 :
						  (r - g) / C + 4
				);
			H = ((H + 360) % 6) * 60 / 360;
			S = C == 0 ? 0 : C / V;
			return {h: H||1, s: S, b: V, a: a||1};
		},
		
		HueToRGB: function (p, q, h) {
			if (h < 0)
				h += 1;
			else if (h > 1)
				h -= 1;

			if ((h * 6) < 1)
				return p + (q - p) * h * 6;
			else if ((h * 2) < 1)
				return q;
			else if ((h * 3) < 2)
				return p + (q - p) * ((2 / 3) - h) * 6;
			else
				return p;
		},
	
		// a set of RE's that can match strings and generate color tuples.
		// from John Resig color plugin
		// https://github.com/jquery/jquery-color/
		stringParsers: [
			{
				re: /rgba?\(\s*(\d{1,3})\s*,\s*(\d{1,3})\s*,\s*(\d{1,3})\s*(?:,\s*(\d+(?:\.\d+)?)\s*)?\)/,
				parse: function( execResult ) {
					return [
						execResult[ 1 ],
						execResult[ 2 ],
						execResult[ 3 ],
						execResult[ 4 ]
					];
				}
			}, {
				re: /rgba?\(\s*(\d+(?:\.\d+)?)\%\s*,\s*(\d+(?:\.\d+)?)\%\s*,\s*(\d+(?:\.\d+)?)\%\s*(?:,\s*(\d+(?:\.\d+)?)\s*)?\)/,
				parse: function( execResult ) {
					return [
						2.55 * execResult[1],
						2.55 * execResult[2],
						2.55 * execResult[3],
						execResult[ 4 ]
					];
				}
			}, {
				re: /#([a-fA-F0-9]{2})([a-fA-F0-9]{2})([a-fA-F0-9]{2})/,
				parse: function( execResult ) {
					return [
						parseInt( execResult[ 1 ], 16 ),
						parseInt( execResult[ 2 ], 16 ),
						parseInt( execResult[ 3 ], 16 )
					];
				}
			}, {
				re: /#([a-fA-F0-9])([a-fA-F0-9])([a-fA-F0-9])/,
				parse: function( execResult ) {
					return [
						parseInt( execResult[ 1 ] + execResult[ 1 ], 16 ),
						parseInt( execResult[ 2 ] + execResult[ 2 ], 16 ),
						parseInt( execResult[ 3 ] + execResult[ 3 ], 16 )
					];
				}
			}
		],
		template: '<div class="colorpicker dropdown"><input type="text" class="infos" value=""><div class="colorpicker-color"><div class="color-line"></div></div><span class="ok">ok</span><br clear="all"><div class="sets"><div class="st" id="c1">&nbsp;</div><div class="st" id="c2">&nbsp;</div><div class="st" id="c3">&nbsp;</div><div class="st" id="c4">&nbsp;</div><div class="st" id="c5">&nbsp;</div><div class="st" id="c6">&nbsp;</div><div class="st" id="c7">&nbsp;</div><div class="st" id="c8">&nbsp;</div><div class="st" id="c9">&nbsp;</div><div class="st" id="c10">&nbsp;</div><div class="st" id="c11">&nbsp;</div><div class="st" id="c12">&nbsp;</div><div class="st" id="c13">&nbsp;</div><div class="st" id="c14">&nbsp;</div><div class="st" id="c15">&nbsp;</div><div class="st" id="c16">&nbsp;</div><div class="st" id="c17">&nbsp;</div><div class="st" id="c18">&nbsp;</div><div class="st" id="c19">&nbsp;</div><div class="st" id="c20">&nbsp;</div><div class="st" id="c21">&nbsp;</div><div class="st" id="c22">&nbsp;</div><div class="st" id="c23">&nbsp;</div><div class="st" id="c24">&nbsp;</div><div class="st" id="c25">&nbsp;</div><div class="st" id="c26">&nbsp;</div><div class="st" id="c27">&nbsp;</div></div>'+
							'<div class="colorpicker-saturation"><i><b></b></i></div>'+
							'<div class="colorpicker-hue"><i></i></div>'+
							'<div class="colorpicker-alpha"><i></i></div><br>'+
						'</div>'
	};

}( window.jQuery )