<?php
namespace Admin\Repository;

use Admin\Controller\Dto\FormOrderSearchDto;
use Shop\Repository\SqlHandlerRepository;
use Shop\Exception\NotFoundException;
use Common\Lib\DB;

class SqlOrderRepository
{
    private $con;

    public function __construct()
    {
        $this->con = DB::connection();
    }

    public function findCouponByCode($code)
    {
        if (!$code)
        {
            throw new NotFoundException('Not found');
        }

        $r = $this->con->query('SELECT *, UNIX_TIMESTAMP(expire) as expire
                    FROM '.DB_PREFIX.'order_coupon
                    WHERE code = "'.trim($this->con->real_escape_string($code)).'"');

        $return = $r->fetch_assoc();
        if (!$return)
        {
            throw new NotFoundException('Not found coupon');
        }

        return $return;
    }

    public function getList(FormOrderSearchDto $dto, $endLimit = NULL)
    {
        $return = array();
        $where = $limit = '';

        if (!empty($dto->formId)) {
            $where .= ' AND o.formId = '. (int) $dto->formId;
        }
        if (!empty($dto->langId)) {
            $where .= ' AND o.langId = '. (int) $dto->langId;
        }
        if (!empty($dto->domainId)) {
            $where .= ' AND o.domainId = '. (int) $dto->domainId;
        }

        if ($dto->goal) {
            $allRequests = SqlHandlerRepository::$allRequests;
            if ($dto->goal == 'requests') {
                $goalList = $allRequests;
            } else if (in_array($dto->goal, $allRequests)) {
                $goalList[] = 'requests';
            }
            $goalList[] = $dto->goal;
            $where .= ' AND o.goal IN ("'.implode('","', $goalList).'", "")';
        }

        if ($dto->event !== '') {
    		$where .= ' AND o.event = "'. $this->con->real_escape_string($dto->event) .'"';
        }

        if (!empty($dto->couponId)) {
            $where .= ' AND o.couponId = '. (int) $dto->couponId;
        }

        if (!empty($dto->userId) && !empty($dto->partner1Id)) {
        	$where .= ' AND (o.userId = '. (int) $dto->userId .' OR o.partner1Id = '. (int) $dto->partner1Id.')';
        } else {
	        if (!empty($dto->userId)) {
	            $where .= ' AND o.userId = '. (int) $dto->userId;
	        }
	        if (!empty($dto->partner1Id)) {
	            $where .= ' AND o.partner1Id = '. (int) $dto->partner1Id;
	        }
        }
    	if ($endLimit != NULL) {
            $limit = ' LIMIT '. (int) $dto->page .', '. (int) $endLimit;
        }

        $r = $this->con->query('SELECT o.*,
                                       UNIX_TIMESTAMP(o.dateCreate) as dateCreate,
                                       u.userEmail as partnerEmail,
                                       u.userPhone as partnerPhone
                                  FROM '.DB_PREFIX.'order o LEFT JOIN '.DB_PREFIX.'acl_user u ON (o.partner1Id = u.id)
                                 WHERE 1'.$where.'
                                 ORDER BY o.id DESC'.
                                 $limit);

        while ($result = $r->fetch_assoc())
        {
        	$result['partnerLogin'] = $result['partnerEmail'] ? $result['partnerEmail'] : $result['partnerPhone'];
        	$result['orderInfo'] = unserialize($result['orderInfo']);
        	$return[] = $result;
        }

        return $return;
    }

    public function countItems(FormOrderSearchDto $dto)
    {
    	$where = '';
    	if (!empty($dto->formId)) {
            $where .= ' AND formId = '. (int) $dto->formId;
        }
        if (!empty($dto->langId)) {
            $where .= ' AND langId = '. (int) $dto->langId;
        }
        if (!empty($dto->domainId)) {
            $where .= ' AND domainId = '. (int) $dto->domainId;
        }

        if ($dto->goal) {
            $allRequests = SqlHandlerRepository::$allRequests;
            if ($dto->goal == 'requests') {
                $goalList = $allRequests;
            } else if (in_array($dto->goal, $allRequests)) {
                $goalList[] = 'requests';
            }
            $goalList[] = $dto->goal;
            $where .= ' AND goal IN ("'.implode('","', $goalList).'", "")';
        }

        if ($dto->event !== '') {
    		$where .= ' AND event = "'. $this->con->real_escape_string($dto->event) .'"';
        }

        if ($dto->couponId !== '') {
            $where .= ' AND couponId = '. (int) $dto->couponId;
        }

        if ($dto->userId !== '' && $dto->partner1Id !== '') {
        	$where .= ' AND (userId = '. (int) $dto->userId .' OR partner1Id = '. (int) $dto->partner1Id.')';
        } else {
	        if ($dto->userId !== '') {
	            $where .= ' AND userId = '. (int) $dto->userId;
	        }
	        if ($dto->partner1Id !== '') {
	            $where .= ' AND partner1Id = '. (int) $dto->partner1Id;
	        }
        }

        $r = $this->con->query('SELECT count(*) as count FROM '.DB_PREFIX.'order WHERE 1'.$where);
        $result = $r->fetch_assoc();

        return (int) $result['count'];
    }

    public function countActive()
    {
        $r = $this->con->query('SELECT count(*) as count FROM '.DB_PREFIX.'order WHERE event = "submit"');
        $result = $r->fetch_assoc();

        return (int) $result['count'];
    }
}