<?php
namespace Shop\Entity;

use Shop\Entity\Customer;
use Shop\Entity\Product;
use Shop\Entity\Payment;

class Order
{
    private $id;
    private $formId;
    private $product;
    private $customer;
    private $promoCode;
    private $price = 0;
    private $increase = 0;
    private $goal;
    private $event; // 'submit', 'work', 'paid', 'delivery', 'done', 'cancel', 'delete'
    private $newEvent = '';
    private $payment;
    private $coupon = [];
    private $langId = 1;
    private $pageId = 1;
    private $domainId = 1;
    private $dateCreate;
    private $dateUpdate;
    private $files = [];
    private $isErrorSendFiles = false;

    private $recaptchaToken;

    private $isError = 0;
    private $isSend = 0;

    private $addVars = [];
    private $updateVars = [];

    private $orderInfo = [];

    private $redirect = '';
    private $isReload = false;
    private $alert = '';
    private $script = '';

    private $operation; //update, delete

    public function __construct($id,
                                $formId,
                                Customer $customer,
                                Payment $payment,
                                $price,
                                $increase,
                                $goal,
                                $langId,
                                $pageId,
                                $domainId,
                                $orderInfo = []
                               )
    {
        $this->id = $id;
        $this->formId = (int) $formId;
        $this->customer = $customer;
        $this->payment = $payment;
        $this->price = (float) $price;
        $this->increase = (float) $increase;
        $this->goal = $goal;
        $this->langId = (int) $langId;
        $this->pageId = (int) $pageId;
        $this->domainId = (int) $domainId;
        $this->orderInfo = $orderInfo;
        $this->dateCreate = time();
        $this->dateUpdate = time();

    }

    public function id()
    {
        return $this->id;
    }
    public function setId($id)
    {
        $this->id = $id;
    }

    public function name()
    {
        return $this->product->name();
    }

    public function addProduct($id, $name, $price, $quantity)
    {
        $this->product = new Product($id, $name, $price, $quantity);
    }
    public function productId()
    {
        return $this->product->id();
    }
    public function productName()
    {
        return $this->product->name();
    }
    public function productPrice()
    {
        return $this->product->price();
    }
    public function productQuantity()
    {
        return $this->product->quantity();
    }

    public function formId()
    {
        return $this->formId;
    }

    public function goal()
    {
        return $this->goal;
    }

    public function langId()
    {
        return $this->langId;
    }

    public function customer()
    {
        return $this->customer;
    }

    public function payment()
    {
        return $this->payment;
    }
    public function setPayment($payment)
    {
        $this->payment = $payment;
    }

    public function isErrorSendFiles()
    {
        return $this->isErrorSendFiles;
    }
    public function setErrorSendFiles()
    {
        $this->isErrorSendFiles = true;
    }

    public function event()
    {
        return $this->event;
    }
    public function setEvent($event)
    {
        $this->event = $event;
        if ($event === 'delete')
        {
            $this->setOperation('delete');
        }
    }

    public function newEvent()
    {
        $return = $this->newEvent;

        $this->newEvent = '';

        return $return;
    }
    public function setNewEvent($event)
    {
        $this->newEvent = $event;
    }

    public function couponId()
    {
        return isset($this->coupon['id']) ? $this->coupon['id'] : 0;
    }

    public function coupon()
    {
        return $this->coupon;
    }
    public function setCoupon($coupon)
    {
        $this->coupon = $coupon;
    }

    public function promoCode()
    {
        return $this->promoCode;
    }
    public function setPromoCode($promoCode)
    {
        $this->promoCode = $promoCode;
    }

    public function dateCreate()
    {
        return $this->dateCreate;
    }
    public function setDateCreate($date)
    {
        $this->dateCreate = $date;
    }

    public function dateUpdate()
    {
        return $this->dateUpdate;
    }
    public function setDateUpdate($date)
    {
        $this->dateUpdate = $date;
    }

    public function recaptchaToken()
    {
        return $this->recaptchaToken;
    }
    public function setRecaptchaToken($token)
    {
        $this->recaptchaToken = $token;
    }

    public function price()
    {
        $price = $this->product->price();
        if ($price)
        {
            $price = $this->product->quantity() * $price;
            $price += $this->increase;
        }
        else
        {
            $price = $this->price;
        }

        return $price >= 0 ? (float) round($price, 2) : 0;
    }

    public function setCommission($sum, $type)
    {
        $price = $this->price();
        if (!$sum || !$price)
        {
            return;
        }

        if ($type == '%')
        {
            $sum = $price * $sum / 100;
        }

        $sum = round($sum, 2);

        $this->updateOrderInfo('partnerCommissions1', [
            'baseName' => Commissions,
            'value' => (float) $sum
        ]);

        $this->customer()->partner()->setCommission($sum);

    }

    public function addDiscount($sum, $type)
    {
        $price = $this->price();

        if (!$sum || empty($price))
        {
            return;
        }

        if ($type === '%')
        {
            return $this->increase -= $price * $sum / 100;
        }

        return $this->increase -= $sum;
    }

    public function increase()
    {
        return $this->increase;
    }

    public function pageId()
    {
        return $this->pageId;
    }

    public function roistat()
    {
        return isset($this->orderInfo['roistat_visit']['value']) ? $this->orderInfo['roistat_visit']['value'] : '';
    }

    public function domainId()
    {
        return $this->domainId;
    }

    public function isSend()
    {
        return $this->isSend;
    }
    public function setIsSend($isSend)
    {
        $this->isSend = (int) $isSend;
    }

    public function files()
    {
        return $this->files;
    }
    public function setFiles($files)
    {
        $this->files = $files;
    }

    public function redirect()
    {
        return $this->redirect;
    }
    public function setRedirect($redirect)
    {
        $this->redirect = $redirect;
    }

    public function isReload()
    {
        return $this->isReload;
    }
    public function setIsReload($isReload)
    {
        $this->isReload = $isReload;
    }

    public function alert()
    {
        return $this->alert;
    }
    public function addAlert($alert)
    {
        if ($this->alert && $alert)
        {
            $this->alert .= "\n";
        }
        $this->alert .= $alert;
    }

    public function script()
    {
        return $this->script;
    }
    public function addScript($script)
    {
        $this->script .= $script." ";
    }

    public function orderInfo($key = null)
    {
        if ($key === null) {
            return $this->orderInfo;
        }
        return isset($this->orderInfo[$key]) ? $this->orderInfo[$key] : null;
    }

    public function updateOrderInfo($key, $value)
    {
        $this->orderInfo[$key] = $value;
        if (isset($value['error']))
        {
            $this->setError(1);
        }
    }

    public function removeOrderInfo($key)
    {
        if (isset($this->orderInfo[$key])) unset($this->orderInfo[$key]);
    }

    public function error()
    {
        return $this->isError;
    }
    public function setError($isError)
    {
        $this->isError = (int) $isError;
    }

    public function vars()
    {
        $vars = [
            '%timer%'=>'',
            '%paymentMethod%'=>'',
            '%payStatus%'=> paymentWaitPaid,
            '%orderPage%'=>'',
            '%reftail%'=>'',
            '%utm%'=>'',
            '%utm_source%' => '',
            '%utm_medium%' => '',
            '%utm_campaign%' => '',
            '%utm_content%' => '',
            '%utm_term%' => ''
        ];

        foreach($this->orderInfo as $key => $item)
        {
            $vars['%'.$key.'%'] = $item['value'];
        }

        $vars['%orderId%'] = $this->id;
        $vars['%orderName%'] = $this->name();
        $vars['%price%'] = $this->price();

        $vars['%productId%'] = $this->product->id();
        $vars['%productName%'] = $this->product->name();
        $vars['%productPrice%'] = $this->product->price();
        $vars['%productQuantity%'] = $this->product->quantity();

        $vars['%userName%'] = $this->customer->name();
        if (empty($vars['%userEmail%']))
        {
            $vars['%userEmail%'] = $this->customer->email();
        }
        if (empty($vars['%userPhone%']))
        {
            $vars['%userPhone%'] = $this->customer->phone();
        }
        $vars['%userPassword%'] = $this->customer->password();
        $vars['%userAccountMoney%'] = $this->customer->money();
        $vars['%promoCode%'] = $this->customer->promoCode();

        $vars['%partnerName%'] = $this->customer->partner()->name();
        $vars['%partnerEmail%'] = $this->customer->partner()->email();
        $vars['%partnerPhone%'] = $this->customer->partner()->phone();
        $vars['%partnerAccountMoney%'] = $this->customer->partner()->money();
        $vars['%partnerCommissions%'] = $this->customer->partner()->commission();

        $vars['%dateTime%'] = date('j F Y H:i:s', $this->dateCreate);
        $vars['%date%'] = date('j F Y', $this->dateCreate);

        if (isset($this->orderInfo['payStatus']) && !isset($this->orderInfo['paymentMethod'])) {
            $vars['%paymentMethod%'] = $this->orderInfo['payStatus']['baseName'];
        }

        $vars['%orderInfo%'] = $this->htmlOrderInfo();

        $utms = false;
        if (!empty($vars['%utm%']));
        {
            $utms = explode("; ", $vars['%utm%']);
            foreach($utms as $value) {
                if (empty($value)) continue;
                $value = explode(" = ", $value);
                if (!empty($value[1]) && !empty($value[0])) {
                    $vars['%utm_'.$value[0]."%"] = $value[1];
                }
            }
        }

        return array_merge($vars + $this->addVars, $this->updateVars);
    }

    public function addVars($newData)
    {
        $this->addVars = array_merge($this->addVars, $newData);
    }

    public function updateVars($newData)
    {
        if ($this->vars() != $newData)
        {
            $this->updateVars = $newData;
        }
    }

    private function htmlOrderInfo()
    {
        $return = '<div style="padding-top:5px;"><b>№</b> ' . $this->id() . '</div>';
        $return .= '<div style="padding-top:5px;"><b>'.baseName.'</b>: ' . $this->name() . '</div>';

        if ($this->price())
        {
            $return .= '<div style="padding-top:5px;"><b>'. price .'</b>: ' . $this->price() . '</div>';
        }

        $userPhone = $userEmail = false;

		foreach($this->orderInfo as $key=>$item)
        {
		    if (isset($item['userPassword']) ||
                !empty($item['skips']) ||
                $item['value'] === '' ||
                $key === 'orderNumber' ||
                $key === 'goodName' ||
                $key === 'price'
               )
            {
                continue;
            }

            $return .= '<div style="padding-top:5px;'.(isset($item['error']) ? 'color:red;' : '').'">';

            if (!empty($item['baseName']))
            {
		        $return .= '<b>'.$item['baseName'].'</b>: ';
            }

            if ($key === 'reftail')
            {
                $host = parse_url($item['value'], PHP_URL_HOST);
                if ($host)
                {
                    $item['value'] = $host;
                }
            }

            $return .= $item['value'].'</div>';
            if (isset($item['userPhone']))
            {
                $userPhone = true;
            }
            else if (isset($item['userEmail']))
            {
                $userEmail = true;
            }
		}

        if (!$userEmail && $this->customer()->email())
        {
            $return .= '<div style="padding-top:5px;"><b>E-mail</b>: <a href="mailto:' . $this->customer()->email() . '">' . $this->customer()->email() . '</a></div>';
        }

        if (!$userPhone && $this->customer()->phone())
        {
            $return .= '<div style="padding-top:5px;"><b>'. Phone .'</b>: <a href="tel:' . $this->customer()->phone() . '">' . $this->customer()->phone() . '</a></div>';
        }

        if ($this->customer()->partner()->id() && $this->customer()->partner()->id() != $this->customer()->id())
        {
            $return .= '<div style="padding-top:5px;"><b>'. Partner .'</b>: <a href="'.siteProtocol.'://'.DOMAIN.ADMIN_DIR.'?service=user_ACL&action=edit&id='.$this->customer()->partner()->id().'" target="_blank">'.$this->customer()->partner()->login().'</a></div>';
        }

		return $return;
    }

    public function operation()
    {
        return $this->operation;
    }
    public function setOperation($operation)
    {
        $this->operation = $operation;
    }

}