<?php
namespace Shop\Usecase\Dto;

use Shop\Exception\ErrorMessageException;
use Admin\Repository\SqlFormRepository;
use main;

class FormDto
{
    public $orderId = 0;
    public $formId = 0;
    public $domainId = 1;
    public $pageId = 1;
    public $pagePath = '';
    public $langId = 1;
    public $price = 0;
    public $buttonId = 0;
    public $productId = 0;
    public $productName = '';
    public $quantity = 1;
    public $increase = 0;
    public $login = '';
    public $password = '';
    public $userEmail = '';
    public $userPhone = '';
    public $userName = '';
    public $paymentId = 0;
    public $promoCode = '';
    public $partnerCode = '';
    public $goal = '';
    public $flist = [];
    public $orderInfo = [];
    public $recaptchaToken;
    public $ip;
    public $timers = [];
    public $roistat_visit = '';

    public $reftail;
    public $utm;

    private $fileNames = [];

    public static function fromRequest($request)
    {
        return (new self())->load($request);
    }

    public function load($request)
    {
        if (!isset($request['formId']) ||
            !isset($request['isJavaScript']) ||
            $request['fields'][1] !== '' ||
		    ($request['fields'][0] != md5(DOMAIN.$request['formId']).(date('n')-1) && $request['fields'][0] != md5(DOMAIN.$request['formId']).date('n')))
        {
            throw new ErrorMessageException("Bot error");
        }

        $uprice = false;
        $loginId = $passwordId = $emailId = $phoneId = $userNameId = 0;
        $this->orderId = uniqid('', true);
        $this->formId = (int) $request['formId'];
        $this->productId = (int) $request['productId'];
        $this->buttonId = (int) $request['buttonId'];
        $this->pageId = $request['id'];
        $this->langId = CUR_LANG_ID;

        if (isset($_COOKIE['timers']))
        {
            $this->timers = $_COOKIE['timers'];
        }

        if (isset($_GET['pr']))
        {
            $this->partnerCode = $this->promoCode = $_GET['pr'];
        }
        else if (isset($_COOKIE['prCode']))
        {
            $this->partnerCode = $this->promoCode = $_COOKIE['prCode'];
        }

        if (isset($_POST['recaptchaToken']))
        {
            $this->recaptchaToken = $_POST['recaptchaToken'];
        }


        if (isset($_COOKIE['roistat_visit']))
        {
            $this->roistat_visit = $_COOKIE['roistat_visit'];
        }

        if (!empty($_SERVER["HTTP_CF_CONNECTING_IP"]) && filter_var($_SERVER["HTTP_CF_CONNECTING_IP"], FILTER_VALIDATE_IP))
        {
            $this->ip = $_SERVER["HTTP_CF_CONNECTING_IP"];
        }
        else
        {
            $this->ip = $_SERVER['REMOTE_ADDR'];
        }

        $IDN = new \idna_convert(['idn_version' => 2008]);

        if (isset($_COOKIE['reftail3']))
        {
            $reftail = parse_url($_COOKIE['reftail3'], PHP_URL_HOST);
            if (DOMAIN != $reftail &&
                DOMAIN != 'www.'.$reftail &&
                $IDN->decode(DOMAIN) != $reftail &&
                $IDN->decode(DOMAIN) != 'www.'.$reftail)
            {
                $this->reftail = $IDN->decode($reftail);
            }
        }

        if (!empty($_COOKIE['utm'])) {
            $this->utm = $_COOKIE['utm'];
        }

        if (!empty($GLOBALS['domain']['domainId'])) {
            $this->domainId = $GLOBALS['domain']['domainId'];
        }

        $this->pagePath = siteProtocol.'://'.$IDN->decode(DOMAIN).BASE_DIR.'/'.$_REQUEST['path'];

        if (!$this->productId && $this->buttonId)
        {
            $this->productId = $this->buttonId;
        }

        $product = main::mapper('block_core')->findById($this->productId);
        if (empty($product['settings']['payLink']))
        {
            $product = false;
            $this->productId = 0;
        }
        if (isset($product['settings']['productName'])) {
			$this->productName = $product['settings']['productName'];
        }

        $formRepo = new SqlFormRepository();
        $form = $formRepo->getById($this->formId);
        $this->goal = $form['goal'];

        $this->flist = $formRepo->getFields($this->formId);
        foreach($this->flist AS $id => $item)
        {
            $fname = '';
            if (isset($item['settings']['fname']))
            {
                $fname = $item['settings']['fname'];
            }

            if ($item['service'] == 'submit_forms' || $item['service'] == 'button_forms')
            {
                unset($this->flist[$id]);
                continue;
            }

            if (isset($item['settings']['placeholder']) && $item['baseName'] == $request['fields'][$id])
            {
                $request['fields'][$id] = '';
            }

            if (isset($request['fields'][$id])) {
                if ($item['service'] == 'checkbox_forms') {
                    foreach($request['fields'][$id] as $key=>$value) {
                        $request['fields'][$id][$key] = trim(preg_replace('/\s{2,}/u', ' ', preg_replace('/([^\s]{70})/u' , '$1 ', $value)));
                    }
                } else if ($item['service'] == 'phone_forms') {
                    if ($request['fields'][$id] == $item['text1'])
                    {
                        $request['fields'][$id] = '';
                    }
					$request['fields'][$id] = trim(str_replace(array('(',')','-',' '), '', preg_replace('/\s{2,}/u', ' ', preg_replace('/([^\s]{70})/u' , '$1 ', $request['fields'][$id]))));
                } else {
                    $request['fields'][$id] = trim(preg_replace('/\h{2,}/u', ' ', preg_replace('/([^\s]{70})/u' , '$1 ', $request['fields'][$id])));
                }
            }
            if ($item['service'] == 'email_forms')
            {
                $emailId = $id;
            }
            else if ($item['service'] == 'phone_forms')
            {
                $phoneId = $id;
            }
            else if ($item['service'] == 'name_forms')
            {
                $userNameId = $id;
            }
            else if ($item['service'] == 'passwordInput_forms')
            {
                $passwordId = $id;
            }
            else if ($item['service'] == 'login_forms')
            {
                $loginId = $id;
            }

	    }

        foreach($this->flist AS $id => $item)
        {
            $fname = '';
            if (isset($item['settings']['fname']))
            {
                $fname = $item['settings']['fname'];
            }
            if ($item['service'] == 'email_forms')
            {
                if ($request['fields'][$id] == '@') {
                    $request['fields'][$id] = '';
                }
                $string = preg_replace("/[^\@]/ui", "", $request['fields'][$id]);
                if (!$string) {
                    $request['fields'][$id] = '';
                }
            }

			if ($item['service'] == 'file_forms' &&
                main::rule('isUploadFile_core', $id, array($item['baseName'], 'fields'), $id) &&
                !empty($item['settings']['accept']))
			{
    	        main::rule('uploadFileExt_core', $id, $item['settings']['accept'], array($item['baseName'], 'fields'), $id);
			}

	        if (!empty($item['settings']['requires']))
            {
        	    if ($item['service'] == 'file_forms') {
        		    main::rule('requireUploadFile2_core', $id, array($item['baseName'], 'fields'), $id);
        		} else {
        		    main::rule('require_core', $id, array($item['baseName'], isset($request['fields'][$id]) ? $request['fields'][$id] : null));
        		}
	        }

	        if ($item['service'] == 'priceInput_forms') {
	        	$uprice = $request['fields'][$id] = (float) str_replace(',','.', preg_replace('/\s{1,}/u', '', $request['fields'][$id]));
                unset($this->flist[$id]);
                continue;
	        }

            if ($item['service'] == 'promo_forms') {
                $this->promoCode = $request['fields'][$id];
            }

	        if (!empty($item['settings']['multiply']))
            {
                if (empty($request['fields'][$id]))
                {
                    $request['fields'][$id] = 1;
                }

                $quantity = round(str_replace(',','.', preg_replace('/\s{1,}/u', '', $request['fields'][$id])), 3);
                if ($quantity)
                {
                    $this->quantity = (float) $quantity;
                }

	        }

            if ($item['service'] == 'checkboxOne_forms' && isset($request['fields'][$id]) && !empty($item['settings']['increase'])) {
	            $this->increase += (float) $item['settings']['increase'];
	        }

            if ($item['service'] == 'paymentRadio_forms' || $item['service'] == 'paymentSelect_forms') {
                $this->paymentId = (int) $request['fields'][$id];
            }

            if ($fname == 'productName')
            {
                $this->productName = $request['fields'][$id];
            }
        }

        if ($emailId)
        {
            $this->userEmail = $request['fields'][$emailId];
        }

        if ($phoneId)
        {
            $this->userPhone = $request['fields'][$phoneId];
        }

        if ($userNameId)
        {
            $this->userName = $request['fields'][$userNameId];
        }

        if ($loginId)
        {
            $this->login = $request['fields'][$loginId];
        }

        if ($passwordId)
        {
            $this->password = $request['fields'][$passwordId];
        }

        if (empty($this->userEmail) && empty($this->userPhone))
        {
            if (main::$user['userEmail']) {
                $this->userEmail = main::$user['userEmail'];
            }
            if (main::$user['userPhone']) {
                $this->userPhone = main::$user['userPhone'];
            }
            if (main::$user['userName'] && empty($this->userName)) {
                $this->userName = main::$user['userName'];
            }
        } elseif (empty($this->userEmail)) {
            if (main::$user['userEmail'] && main::$user['userPhone'] == $this->userPhone) {
            	$this->userEmail = main::$user['userEmail'];
                if (main::$user['userName'] && empty($this->userName)) {
                    $this->userName = main::$user['userName'];
                }
            }
        }  elseif (empty($this->userPhone)) {
            if (main::$user['userPhone'] && main::$user['userEmail'] == $this->userEmail) {
            	$this->userPhone = main::$user['userPhone'];
            	if (main::$user['userName'] && empty($this->userName)) {
                    $this->userName = main::$user['userName'];
                }
            }
        }

        if (isset($product['settings']['price']))
        {
            $this->price = (float) $product['settings']['price'];
        }

        if ($uprice !== false)
        {
            if (!$this->price || ($this->quantity === 1 && $this->increase === 0))
            {
			    $this->price = $uprice;
            }
		}

        if ($this->productName === '')
        {
            $this->productName = $form['formName'];
        }

        $this->createContent();

        return $this;

    }

    private function createContent()
    {
        $content = [];
        foreach($this->flist as $id => $item)
        {
            $fname = 'var'.$id;
            if (!empty($item['settings']['fname']) && !isset($content[$item['settings']['fname']]))
            {
                $fname = $item['settings']['fname'];
            }

            if ($item['service'] == 'paymentSelect_forms' ||
                $item['service'] == 'paymentRadio_forms' ||
                $fname == 'productName')
            {
               continue;
            }

            if ($item['service'] == 'file_forms')
            {
				if (is_array($_FILES['fields']['name'][$id]))
				{
					$files = [];
					foreach($_FILES['fields']['name'][$id] as $key => $val) {

						if (empty($val)) {
							continue;
						}

						$files[] = $this->fileUpload(
							$this->orderId,
							$_FILES['fields']['tmp_name'][$id][$key],
							$val,
							$_FILES['fields']['error'][$id][$key]
						);

					}
					$files = implode(', ', $files);

				}
				else
				{
					if (empty($_FILES['fields']['name'][$id])) {
						continue;
					}

					$files = $this->fileUpload(
						$this->orderId,
						$_FILES['fields']['tmp_name'][$id],
						$_FILES['fields']['name'][$id],
						$_FILES['fields']['error'][$id]
					);
				}

				if (empty($files)) continue;

		    	$content[$fname] = [
                    'baseName' => $item['baseName'],
                    'isFile' => 1,
                    'value' => $files
                ];

		    }
            elseif ($item['service'] == 'textArea_forms')
            {
		    	$content[$fname] = [
                    'baseName' => $item['baseName'],
					'value' => nl2br(escape($_REQUEST['fields'][$id]))
                ];

		    }
            elseif ($item['service'] == 'checkbox_forms')
            {
                $shpch = array();
		        if (!empty($_REQUEST['fields'][$id]) && is_array($_REQUEST['fields'][$id]))
                {
                    $l = explode("\n", str_replace("\r", '', $item['text1']));
			        foreach($_REQUEST['fields'][$id] as $kch=>$vch) {
			        	if (isset($l[$kch])) {
		            	   $shpch[] = preg_replace('/\s{2,}/u', ' ', strip_tags($l[$kch]));
			        	}
	                }
                }

		        $content[$fname] = [
                    'baseName' => $item['baseName'],
					'value' => !empty($shpch) ? implode(', ', $shpch) : '-'
                ];

		    }
            elseif ($item['service'] == 'radio_forms')
            {
                $l = explode("\n", str_replace("\r", '', $item['text1']));
                $countj = count($l);
                $baseName = '';
                for($j = 0; $j < $countj; $j++)
                {
                    if ($_REQUEST['fields'][$id] == $j)
                    {
                        $baseName = preg_replace('/\s{2,}/u', ' ', strip_tags($l[$j]));
                    }
                }

                $content[$fname] = [
                    'baseName' => $item['baseName'],
			    	'value' => $baseName
                ];

		    }
            elseif ($item['service'] == 'select_forms')
            {
		        $l = explode("\n", str_replace("\r", '', $item['text1']));
		        $countj = count($l);
		        $baseName = '';
		        if ($_REQUEST['fields'][$id] !== '')
                {
		       	    if (!empty($item['text1']))
                       {
				        for($j = 0; $j < $countj; $j++)
                        {
                            if ($_REQUEST['fields'][$id] == $j) {
                                $baseName = preg_replace('/\s{2,}/u', ' ', strip_tags($l[$j]));
                            }
				        }
		       	    } else {
		       	   	    $baseName = preg_replace('/\s{2,}/u', ' ', strip_tags($_REQUEST['fields'][$id]));
		       	    }
		        }

                $content[$fname] = [
                    'baseName' => $item['baseName'],
                    'value' => $baseName
                ];

		    }
            elseif ($item['service'] == 'checkboxOne_forms')
            {
		    	$content[$fname] = [
                    'baseName' => $item['baseName'],
					'value' => !empty($_REQUEST['fields'][$id]) ? '+' : '-'
                ];
		    }
            else
            {
				$content[$fname] = [
                    'baseName' => $item['baseName'],
					'value' => escape($_REQUEST['fields'][$id])
                ];

                if (isset($item['skips']))
                {
                    $content[$fname]['skips'] = 1;
                }

				if ($item['service'] == 'passwordInput_forms')
                {
                    $content[$fname]['userPassword'] = 1;
                }
                else if ($item['service'] == 'phone_forms')
                {
                    $content[$fname]['userPhone'] = 1;
                }
                else if ($item['service'] == 'email_forms')
                {
                    $content[$fname]['userEmail'] = 1;
                }
		    }

		}

        $content['orderPage'] = [
            'baseName' => ordersPage,
            'value' => $this->pagePath
        ];

        if ($this->reftail)
        {
            $content['reftail'] = [
                'baseName' => refTail,
                'value' => $this->reftail
            ];
        }

        if ($this->utm)
        {
            $content['utm'] = [
                'baseName' => 'Utm',
                'value' => preg_replace('/([^\s]{70})/u' , '$1 ', str_replace('utm_', '', $this->utm))
            ];
        }

        if ($this->roistat_visit)
        {
            $content['roistat_visit'] = [
                'baseName' => 'Roistat',
                'value' => $this->roistat_visit
            ];
        }

        $content['IP'] = [
            'baseName' => 'IP',
            'value' => $this->ip
        ];

		$this->orderInfo = $content;

	}

	private function fileUpload($orderId, $tmp_name, $fileName, $error) {

		$fileName = $this->trimFile($fileName);

		try {

			$file = new \fileUploaded($tmp_name,
									$fileName,
									$error);

			$file = $file->move(FILES_S.'orders/'.$orderId, $fileName.'.txt');

			return basename($file->getFilename(), ".txt");

		} catch (\Throwable $e) {
            throw new ErrorMessageException("file upload error");
        }
	}

    private function trimFile($name) {
        $name = trim($this->basename(stripslashes($name)), ".\x00..\x20");
        $name = preg_replace('/([^a-z0-9\!\(\)\.\-\_]+)/ui', '', $name);
        if ($name[0] == '.') $name = microtime(true).$name;
        if (!$name) $name = str_replace('.', '-', microtime(true));
		if (isset($this->fileNames[$name])) $name = count($this->fileNames).$name;
		$this->fileNames[$name] = 1;
        return $name;
    }

    private function basename($filepath, $suffix = '') {
        $splited = preg_split('/\//', rtrim ($filepath, '/ '));
        return substr(basename('X'.$splited[count($splited)-1], $suffix), 1);
    }

}