<?php
namespace Shop\Usecase;

use Shop\Entity\Order;
use Shop\Entity\Product;
use Shop\Entity\Customer;
use Shop\Entity\Partner;
use Shop\Repository\SqlOrderRepository;
use main;

class GetOrderByIdUsecase
{
    private $orderRep;

    public function __construct()
    {
        $this->orderRep = new SqlOrderRepository();
    }

    public function run($orderId)
    {
        $vars = [];
        $item = $this->orderRep->findOrderById($orderId);

        $p = main::mapper('page_core')->findById($item['pageId']);

        $userStatus = 1;
        $langId = $item['langId'];
        $userName = $promoCode = $login = $password = $ip = '';
        $userId = $saleCount = $isPartner = $partnerId = $money = $commissions = 0;
        $userGroups = false;
        $userEmail = $item['userEmail'];
        $userPhone = $item['userPhone'];

        if ($item['userId'])
        {
            $user = $this->orderRep->getCustomerById($item['userId']);
        }
        else if ($userEmail || $userPhone)
        {
            $user = $this->orderRep->getCustomerByEmailAndPhone($userEmail, $userPhone);
        }

        if ($user)
        {
            $userId  = $user['id'];
            $money = $user['money'];
            $saleCount = $user['promo1SaleCount'];
            $isPartner = $user['isPartner'];
            $partnerId = $user['partnerId'];
            $userName = $user['userName'];
            $userStatus = (int) $user['status'];
            $userGroups = $user['groups'];
            $promoCode = $user['promoCode'];
            $password = $user['uPassword'];
            if ($user['langId'])
            {
                $langId = $user['langId'];
            }
        }

        if ($item['partner1Id']) {
            $partnerId = $item['partner1Id'];
        }

        foreach($item['orderInfo'] as $info)
        {
            if (isset($info['userPassword']))
            {
                $password = html_entity_decode($info['value'], ENT_QUOTES, 'UTF-8');
            }
        }

        if (isset($item['orderInfo']['userName']))
        {
            $userName = $item['orderInfo']['userName']['value'];
        }

        if (isset($item['orderInfo']['IP']))
        {
            $ip = $item['orderInfo']['IP']['value'];
        }

        if (isset($item['orderInfo']['partnerCommissions1']))
        {
            $commissions = $item['orderInfo']['partnerCommissions1']['value'];
        }

        $payment = $this->orderRep->findPaymentById($item['paymentId'], $item['paymentStatus']);

        $ord2 = new Order(
            $item['id'],
            $item['formId'],
            new Customer($userId,
                         $langId,
                         $userName,
                         $userEmail,
                         $userPhone,
                         $login,
                         $password,
                         $money,
                         $promoCode,
                         $saleCount,
                         $isPartner,
                         $this->partner($userId,
                                        $userName,
                                        $userEmail,
                                        $userPhone,
                                        $money,
                                        $saleCount,
                                        $commissions,
                                        $isPartner,
                                        $partnerId),
                         $userGroups,
                         $ip,
                         $userStatus),
            $payment,
            $item['price'],
            $item['increase'],
            $item['goal'],
            $item['langId'],
            $item['pageId'],
            $item['domainId'],
            $item['orderInfo']
        );

        $ord2->addProduct($item['blockId'], $item['orderName'], $item['productPrice'], $item['quantity']);
        $ord2->setEvent($item['event']);
        $ord2->setDateCreate($item['dateCreate']);
        $ord2->setDateUpdate($item['dateUpdate']);
        $ord2->setIsSend($item['isSend']);
        $ord2->setOperation('update');

        $ord2->setFiles(
            selectDirFiles(FILES_S.'orders/'.$ord2->id())
        );

        $pvars = main::mapper('page_core')->getVars($p['id'], $p['templateId'], $p['domainId']);
        if (!empty($pvars))
        {
            foreach($pvars[0] as $key => $val)
            {
                $vars[$val] = $pvars[1][$key];
            }
            $ord2->addVars($vars);
        }

        try
        {
            $coupon = $this->orderRep->findCouponById($item['couponId']);
            $ord2->setCoupon($coupon);
        }
        catch (\Exception $e) {}

        return $ord2;

    }

    function partner($id, $name, $email, $phone, $money, $saleCount, $commissions, $isPartner, $partnerId)
    {
        if (!$isPartner)
        {
            $id = $money = $saleCount = 0;
            $name = $email = $phone = '';
            $puser = false;

            if ($partnerId)
            {
                $puser = $this->orderRep->getPartnerById($partnerId);
                if (!empty($puser) && $puser['isPartner'])
                {
                    $id = $puser['id'];
                    $name = $puser['userName'];
                    $email = $puser['userEmail'];
                    $phone = $puser['userPhone'];
                    $money = $puser['money'];
                    $saleCount = $puser['promo1SaleCount'];
                }

            }

        }

        return new Partner(
                            $id,
                            $name,
                            $email,
                            $phone,
                            $money,
                            $saleCount,
                            $commissions
                        );

    }

}