<?php
namespace Shop\Usecase;

use Shop\Usecase\Dto\FormDto;
use Shop\Entity\Order;
use Shop\Entity\Customer;
use Shop\Entity\Partner;
use Shop\Exception\ErrorMessageException;
use Shop\Repository\SqlOrderRepository;
use main;

class GetOrderFromFormUsecase
{
    private $dto;
    private $orderRep;

    public function __construct(FormDto $dto)
    {
        $this->dto = $dto;
        $this->orderRep = new SqlOrderRepository();
    }

    public function run()
    {
        $vars = [];
        $p = main::mapper('page_core')->findById($this->dto->pageId);
        $pvars = main::mapper('page_core')->getVars($this->dto->pageId, $p['templateId'], $p['domainId']);
        if (!empty($pvars))
        {
            $this->dto->productName = str_replace($pvars[0], $pvars[1], $this->dto->productName);

            foreach($pvars[0] as $key => $val)
            {
                $vars[$val] = $pvars[1][$key];
            }
        }

        $payment = $this->orderRep->findPaymentById($this->dto->paymentId);

        $ord2 = new Order(
            $this->dto->orderId,
            $this->dto->formId,
            $this->customer(),
            $payment,
            0,
            $this->dto->increase,
            $this->dto->goal,
            $this->dto->langId,
            $this->dto->pageId,
            $this->dto->domainId,
            $this->dto->orderInfo
        );

        $ord2->addProduct($this->dto->productId, $this->dto->productName, $this->dto->price, $this->dto->quantity);

        $ord2->setFiles(
            selectDirFiles(FILES_S.'orders/'.$ord2->id())
        );

        $ord2->setPromoCode($this->dto->promoCode);
        $ord2->setRecaptchaToken($this->dto->recaptchaToken);
        $ord2->addVars($vars);

        try
        {
            $coupon = $this->orderRep->findCouponByCode($ord2->promoCode());
            $ord2->setCoupon($coupon);
        }
        catch (\Exception $e) {}

        return $ord2;

    }

    function customer()
    {
        $user = false;
        $userId = $saleCount = $isPartner = $partnerId = $money = 0;
        $langId = $this->dto->langId;
        $userName = $userEmail = $userPhone = $login = $password = $promoCode = '';
        $userGroups = false;

        if ($this->dto->userEmail || $this->dto->userPhone)
        {
            $user = $this->orderRep->getCustomerByEmailAndPhone($this->dto->userEmail, $this->dto->userPhone);
        }
        else if ($this->dto->login)
        {
            $user = $this->orderRep->getCustomerByEmailAndPhone($this->dto->login, $this->dto->login);
        }
        else if (main::$user['id'])
        {
            $user = main::$user;
        }


        if ($user)
        {
            if (!$user['status'])
            {
                throw new ErrorMessageException(($user['userEmail'] ? $user['userEmail'] : $user['userPhone']).' blocked');
            }
            $userId = $user['id'];
            $langId = $user['langId'];
            $money = $user['money'];
            $saleCount = $user['promo1SaleCount'];
            $isPartner = $user['isPartner'];
            $partnerId = $user['partnerId'];
            $password = $user['uPassword'];
            $userName = $user['userName'];
            $userEmail = $user['userEmail'];
            $userPhone = $user['userPhone'];
            $userGroups = $user['groups'];
            $promoCode = $user['promoCode'];
        }

        if ($this->dto->userName)
        {
            $userName = $this->dto->userName;
        }

        if ($this->dto->login)
        {
            $login = $this->dto->login;
        }

        if ($this->dto->password)
        {
            $password = $this->dto->password;
        }

        if ($this->dto->userEmail)
        {
            $userEmail = $this->dto->userEmail;
        }

        if ($this->dto->userPhone)
        {
            $userPhone = $this->dto->userPhone;
        }

        $customer = new Customer(
                                 $userId,
                                 $langId,
                                 $userName,
                                 $userEmail,
                                 $userPhone,
                                 $login,
                                 $password,
                                 $money,
                                 $promoCode,
                                 $saleCount,
                                 $isPartner,
                                 $this->partner($userId,
                                                $userName,
                                                $userEmail,
                                                $userPhone,
                                                $money,
                                                $saleCount,
                                                $isPartner,
                                                $partnerId),
                                 $userGroups,
                                 $this->dto->ip
                                );

        $customer->setTimers($this->dto->timers);

        return $customer;
    }

    function partner($id, $name, $email, $phone, $money, $saleCount, $isPartner, $partnerId)
    {
        if (!$isPartner)
        {
            $id = $money = $saleCount = 0;
            $name = $email = $phone = '';
            $puser = false;

            if ($this->dto->promoCode)
            {
                $puser = main::mapper('user_ACL')->getUserByPromo($this->dto->promoCode);
            }

            if (empty($puser) && $this->dto->partnerCode)
            {
                $puser = main::mapper('user_ACL')->getUserByPromo($this->dto->partnerCode);
            }

            if (empty($puser) && $partnerId)
            {
                $puser = $this->orderRep->getPartnerById($partnerId);
            }

            if (!empty($puser) && $puser['isPartner'])
            {
                $id = $puser['id'];
                $name = $puser['userName'];
                $email = $puser['userEmail'];
                $phone = $puser['userPhone'];
                $money = $puser['money'];
                $saleCount = $puser['promo1SaleCount'];
            }
        }

        return new Partner($id,
                           $name,
                           $email,
                           $phone,
                           $money,
                           $saleCount);

    }

}