<?php
namespace Shop\Usecase\Handler;

use Shop\Entity\Order;
use Shop\Exception\NotFoundException;

class CouponUsecase
{
    private $ord2;
    private $handler;

    public function __construct(Order $ord2, $handler = [])
    {
        $this->ord2 = $ord2;
        $this->handler = $handler;
    }

    public function run()
    {
        if (!$this->ord2->couponId() || $this->ord2->orderInfo('dis'.$this->handler['id']))
        {
            return;
        }

        $promo = $this->ord2->coupon();

        $this->tryAccess();
        $this->tryExpire($promo);

        $this->ord2->customer()->setCouponCounterIncrease('Order');

        if (empty($promo['discount']))
        {
            throw new NotFoundException('Not found');
        }

        $this->ord2->addDiscount($promo['discount'], $promo['discountType']);

        $name = $this->handler['baseName'];
        if (empty($name))
        {
            $name = coupon_settings;
        }
        $this->ord2->updateOrderInfo('dis'.$this->handler['id'], [
            'baseName' => discount.' ('.$name.')',
            'value' => floatval($promo['discount']).$promo['discountType']
        ]);

    }

    public function tryAccess()
    {
        if (isset($this->handler['settings']['payall']))
        {
            return true;
        }

        if (isset($this->handler['settings']['ids']) &&
            in_array($this->ord2->couponId(), $this->handler['settings']['ids']))
        {
            return true;
        }

        throw new NotFoundException('Not found access');

    }

    public function tryExpire($promo)
    {
        if (
            ($promo['isExpire'] && $promo['expire'] <= mktime(0, 0, 0, date("m"), date("d") - 1, date("Y"))) ||
            ($promo['isUsedTime'] && $promo['usedTime'] <= $promo['usedTimeOrderCount']) ||
            ($promo['isMinPrice'] && $promo['minPrice'] > $this->ord2->price())
           )
        {
            throw new NotFoundException('Not found');
        }

    }

}