
class cpEditor {

    pageId = 0;

    mouseClick = {
        left: 0,
        top: 0,
        obj: null
    };

    state = {};
    nodes = [];

    xyPan;
    mainMenu;
    panels;
    pages;
    mobileMenu;
    deleteMenu;
    toolbar;
    shortcutsHelp;
    scrollBar;
    eyeOfGod;
    rulers;
    points;
    move;
    layers;
    grid;

    timerId;

    handleEvent(event) {

        this.state[event.type](event);

    }

    constructor(settings) {

        addClass(document.body, "cpEditor");

        this.pageId = settings.pageId;

        this.xyPan = new cpXyPan(this);
        this.contextMenu = new cpContextMenu(this);
        this.mobileMenu = new cpMobileMenu(this);
        this.deleteMenu = new cpDeleteMenu(this);
        this.toolbar = new cpToolbar(this);
        this.rulers = new cpRulers(this, this.pageId, settings.rules);
        this.move = new cpEmptyMove(this);
        this.scrollBar = new cpScrollBar(this);
        this.eyeOfGod = new cpEyeOfGod(this);
        this.state = new cpEditState(this);
        this.grid = new cpGrid(this, this.pageId, settings.grid);
        this.layers = new cpLayers(this);
        this.mainMenu = new cpMainMenu(this);
        this.pages = new cpPages(this);
        this.panels = new cpPanels(this);
        this.shortcutsHelp = new cpShortcutsHelp(this);
        this.setEvents();
        this.searchSelected();
        window.addEventListener('resize', top.cpResizeWindow);
    }

    changeState(state) {

        this.state.finish();
        return this.state = new state(this);

    }

    getState() {

        return this.state;

    }

    setEvents() {

        window.addEventListener("paste", this);
        window.addEventListener("keyup", this);
        top.window.addEventListener("keyup", this);
        window.addEventListener("keydown", this);
        top.window.addEventListener("keydown", this);
        //fixit - сделать задержку, чтоб слишком часто не вызывалось
        //        и скорее всего надо перенести в state вызов ресайза
        window.addEventListener('resize', this.clearHidden);

        document.addEventListener("contextmenu", this, {capture: true});
        top.cpPageEditorMain.addEventListener("contextmenu", this.cancel);

        let list = document.querySelectorAll(".blockM,.cblockM,.dCnt2");
        for(let i=0, length = list.length; i < length; i++)
        {
            list[i].addEventListener("click", this);
            list[i].addEventListener("touchstart", this, {passive: true});
            list[i].addEventListener("dblclick", this);
            list[i].addEventListener("pointerover", this, {passive: true});
        }

    }

    clickCoords(event, el)
    {
        let obj,
            oldClass = '';

        if (!el)
        {
            el = event.target;
            oldClass = ',.drrrOld';
        }

        if (el.classList.contains("drrr0") || (oldClass && el.classList.contains("drrrOld0")))
        {
            obj = document.getElementById('b'+el.id.substr(1));
        }
        else
        {
            obj = el.closest('.drrr,.droppable' + oldClass);
            if (!obj && el.tagName !== 'HTML' && el.tagName !== 'BODY')
            {
                obj = el.querySelector(".droppable");
            }
        }

        this.mouseClick.left = event.clientX;
        this.mouseClick.top = event.clientY;
        this.mouseClick.obj = null;

        if (obj)
        {
            this.mouseClick.obj = obj;
            this.mouseClick.left = this.mouseClick.left - obj.getBoundingClientRect().left - parseFloat(getComputedStyle(obj).borderLeftWidth) + obj.scrollLeft;
            this.mouseClick.top = this.mouseClick.top - obj.getBoundingClientRect().top - parseFloat(getComputedStyle(obj).borderTopWidth) + obj.scrollTop;
        }

    }

    cancel = (event) => {
        this.executeCommand(
            new cpClearCommand(this)
        );
        this.changeState(cpEditState);
        event.preventDefault();
        event.stopPropagation();
    }

    executeCommand(command) {

        command.execute();

    }

    scroll(obj, wraper = "html,body") {

        $(wraper).stop().animate({scrollTop: obj.getBoundingClientRect().top + scrollY - 50}, 1000);

    }

    searchSelected() {

        let list = document.querySelectorAll(".ui-selected");

        if (!list.length) return;

        this.executeCommand(
            new cpSelectMultiCommand(this, list)
        );

    }

    selectPlus(obj, selectType = '') {

        addClass(obj, "ui-selected");
        this.nodes.push(obj);

        if (!selectType) return;

        let list = parents(obj, ".blockM");
        for (let i = 0, length = list.length; i < length; i++) {
            list[i].style.zIndex=5550;
        }
        obj.style.zIndex = 5550 + parseInt(obj.getAttribute("data-zindex"));

    }

    deepSelect(event) {

        document.body.classList.add("cpClearBefore");

        let obj = document.elementFromPoint(event.clientX, event.clientY);
        if (obj) obj = obj.closest(".blockM,.cblockM,.dCnt2");

        document.body.classList.remove("cpClearBefore");

        this.executeCommand(
            new cpClearCommand(this)
        );

        if (obj == event.currentTarget) {

            if (!hasClass(obj, 'dCnt2')) {
                obj.style.visibility = 'hidden';
            }

            let el = document.elementFromPoint(event.clientX, event.clientY);
            if (el) el = el.closest(".blockM,.cblockM,.dCnt2");

            obj.style.visibility = 'visible';
            if (el) obj = el;

        }

        if (obj && obj != event.currentTarget) {
            let selectType = obj.querySelector("#"+event.currentTarget.id) ? '' : 'selectUp';
            this.executeCommand(
                new cpSelectOneCommand(this, obj, true, selectType)
            );

        }

    }

    clearOne(obj) {

        if (this.nodes.length === 1) {
            this.clearSelects();
            return;
        }

        let index = this.nodes.indexOf(obj);
        if (index !== -1) {
            this.nodes.splice(index, 1);
        }

        removeClass(obj, "ui-selected");
        obj.style.zIndex = "";

        this.layers.clearOne(obj);

    }

    clearSelects()
    {
        $(".ui-resizable").resizable("destroy");
        if (!this.nodes.length) return;

        for(let i=0, length = this.nodes.length; i < length; i++) {
           removeClass(this.nodes[i], "ui-selected");
           this.nodes[i].style.zIndex = "";
        }

        let list = parents(this.nodes[0], ".blockM");
        for (let i = 0, length = list.length; i < length; i++) {
            list[i].style.zIndex='';
        }

        this.nodes = [];

        this.layers.clear();

    }

    clearHidden = () =>
    {
        this.save();

        let i = this.nodes.length;
        while (i--)
        {
            let css = getComputedStyle(this.nodes[i]);
            if (css.display === 'none')
            {
                this.clearOne(this.nodes[i]);
            }
        }
    }

    save() {

        this.move.finish();
        if (this.contextMenu.zindex) {
            this.contextMenu.zindex.finish();
        }

    }

    stopClick(mouseup = false) {

        window.addEventListener("click", this.stopClickEvent, {capture: true});
        window.addEventListener("touchend", this.stopClickEvent, {capture: true});
        if (mouseup) {
            window.addEventListener("mouseup", this.stopClickEvent, {capture: true});
        }
        this.state.stopClick();

    }

    stopClickEvent = (event) => {
        if (event && event.type == 'click') {
            event.stopPropagation();
            event.preventDefault();
        }
        window.removeEventListener("click", this.stopClickEvent, {capture: true});
        window.removeEventListener("touchend", this.stopClickEvent, {capture: true});
        window.removeEventListener("mouseup", this.stopClickEvent, {capture: true});
    }

    autoSave() {

        if (this.timerId) {
            clearInterval(this.timerId);
            this.timerId = null;
        }

        let isMoveSave = this.move.saveIfTime();

        let isZindexSave = 1;
        if (this.contextMenu.zindex) {
            isZindexSave = this.contextMenu.zindex.saveIfTime();
        }

        if (!isZindexSave || !isMoveSave) {
            this.timerId = setInterval(() => this.autoSave(), 400);
        }

    }

    animateHeaderButton(selector) {
        let toogleToolbar = top.document.querySelector(selector);
        toogleToolbar.classList.add('cpHeaderButton--active');
        setTimeout(() => toogleToolbar.classList.remove('cpHeaderButton--active'), 200);
        toogleToolbar.blur();
    }

}
