
class cpEditTextState extends cpState {

    buffer = '';
    buffer2 = null;
    bufferId = null;

    constructor(editor) {

        super(editor);

        window.addEventListener("keypress", this.enterBR);
        window.addEventListener("blur", this.blur);

        editor.xyPan.hide();
        editor.contextMenu.hide();
        editor.mobileMenu.hide();

    }

    init(event, obj) {

        this.editor.executeCommand(
            new cpSelectOneCommand(this.editor, obj)
        );

        addClass(obj, 'cpedit2');

        this.bufferId = obj.id;

        let element = document.querySelectorAll("#"+obj.id+" [contenteditable]");
        if (typeof element[0] !== "undefined") {

            this.buffer = element[0].innerHTML;

            element[0].setAttribute("contenteditable", true);
            element[0].focus({preventScroll: true});

            let caretRange = this.getMouseEventCaretRange(event);
            window.setTimeout(() => this.selectRange(caretRange), 10);

        }

        if (typeof element[1] !== "undefined") {

            this.buffer2 = element[1].innerHTML;
            element[1].setAttribute("contenteditable", true);

        }

    }

    click(event) {

        event.stopPropagation();
        event.preventDefault();

        if (this.bufferId === event.currentTarget.id) return;

        this.editor.executeCommand(
            new cpClearCommand(this.editor)
        );

        let state = this.editor.changeState(cpEditState);
        if (this.isTouch) {
            state.touchstart(event);
        }
        state.click(event);

    }

    contextmenu(event) {

        event.stopPropagation();

        let obj = event.target.closest('.blockM,.cblockM');
        if (obj && this.bufferId === obj.id) return;

        event.preventDefault();

        this.editor.executeCommand(
            new cpClearCommand(this.editor)
        );

        this.editor.changeState(cpEditState);

    }

    keydown(event) {

        //shift || ctrl || metaKey || space
        if (event.which == 16 || event.which == 17 || event.which == 91 || event.which == 32) {
            return;
        }

        super.keydown(event);

    }

    keyup(event) {}

    blur = () => {
        this.editor.changeState(cpEditState);
    }

    finish = () => {

        window.removeEventListener("keypress", this.enterBR);
        window.removeEventListener("blur", this.blur);

        this.save();

        let list = document.querySelectorAll('.cpedit2');
        for(let i = 0, length = list.length; i < length; i++) {
            removeClass(list[i], 'cpedit2');
        }

        let list2 = document.querySelectorAll('[contenteditable="true"]');
        for(let i = 0, length = list2.length; i < length; i++) {
            list2[i].setAttribute("contenteditable", false);
        }

        if (window.getSelection) window.getSelection().removeAllRanges();

    }

    save() {

        let newData2 = null,
            obj = document.querySelectorAll("#"+this.bufferId+" [contenteditable]");

        if (typeof obj[1] !== "undefined") {
            newData2 = obj[1].innerHTML;
        }

        if (this.buffer != obj[0].innerHTML || (newData2 !== null && this.buffer2 != newData2)) {

            let text = {
                "saveSimleText": obj[0].innerHTML.replace(/\u200B/g, "").replace(/(\<br\>)+$/g,"").replace(/(\<br\/\>)+$/g,"").replace(/(\<br \>)+$/g,"").replace(/(\<br \/\>)+$/g,"").replace(/\n/g, "")
            }

            if (newData2 != null) {
                text.text1 = obj[1].innerHTML.replace(/\u200B/g, "").replace(/(\<br\>)+$/g,"").replace(/(\<br\/\>)+$/g,"").replace(/(\<br \>)+$/g,"").replace(/(\<br \/\>)+$/g,"").replace(/\n/g, "");
            }

            $.ajax({type: "POST",
                    url:"./actions.php?id="+ this.editor.pageId +"&blockId="+this.bufferId.substr(1),
                    data: text
            });

        }

    }

    paste(event) {

        event.preventDefault();

        let text = event.clipboardData.getData("text/plain").replace(/\r/g, "").replace(/\n/g, "\n<br>");
        document.execCommand("insertHTML", false, text);
    }

    enterBR(event) {

        if (event.which == 13 || event.which == 10 || (event.target.nodeName == "BUTTON" && event.which == 32)) {
            event.preventDefault();
        } else {
            return true;
        }

        let docFragment = document.createDocumentFragment(),
            newEle;
        if (event.which != 32) {
            newEle = document.createElement("br");
            docFragment.appendChild(newEle);
            docFragment.appendChild(document.createTextNode("\u200B"));
        } else {
            newEle = document.createTextNode("\u0020");
            docFragment.appendChild(newEle);
        }

        let range = window.getSelection().getRangeAt(0);
        range.deleteContents();
        range.insertNode(docFragment);
        range = document.createRange();
        range.setStartAfter(newEle);
        range.collapse(true);

        let sel = window.getSelection();
        sel.removeAllRanges();
        sel.addRange(range);

        return false;

    }

    getMouseEventCaretRange(event) {
        let range, x = event.clientX, y = event.clientY;

        // Try the standards-based way next
        if (document.caretPositionFromPoint) {

            let pos = document.caretPositionFromPoint(x, y);
            range = document.createRange();
            range.setStart(pos.offsetNode, pos.offset);
            range.collapse(true);

        }

        // Next, the WebKit way
        else if (document.caretRangeFromPoint) {

            range = document.caretRangeFromPoint(x, y);

        }

        return range;

    }

    selectRange(range) {

        if (!range || range.startContainer.nodeType !== Node.TEXT_NODE) return;

        if (typeof range.select != "undefined") {

            range.select();

        } else if (typeof window.getSelection != "undefined") {

            let sel = window.getSelection();
            sel.removeAllRanges();
            sel.addRange(range);

        }

    }

}
